function [cnnFeat bboxes_flat] = matcaffe_bbox(frames,bboxes,use_gpu,model_def_file,model_file,device_id,opts)
caffe('reset');

if ~exist('opts','var')
    opts=[];
end
if ~isfield(opts,'isnormalize') || isempty(opts.isnormalize)
    opts.isnormalize=true;
end
if ~isfield(opts,'isvectorize') || isempty(opts.isvectorize)
    opts.isvectorize=true;
end
if ~isfield(opts,'batch_size') || isempty(opts.batch_size)
    opts.batch_size=256;
end

if ~exist('model_def_file','var') || isempty(model_def_file)
    model_def_file = '/home/fanyix/code/caffe-master/models/bvlc_reference_caffenet/fc7.prototxt';
end
if ~exist('model_file','var') || isempty(model_file)
    model_file = '/home/fanyix/code/caffe-master/models/bvlc_reference_caffenet/bvlc_reference_caffenet.caffemodel';
end
if ~exist('device_id','var') || isempty(device_id)
    device_id=1;
end
if ~exist('use_gpu','var') || isempty(use_gpu)
    use_gpu=1;
end


% Adjust the batch size and dim to match with models/bvlc_reference_caffenet/deploy.prototxt
batch_size = opts.batch_size;

% disp(list_im);
if mod(length(frames),batch_size)
    warning(['Assuming batches of ' num2str(batch_size) ' images rest will be filled with zeros'])
end

if ~caffe('is_initialized')
    % init caffe network (spews logging info)
    if ~exist('use_gpu', 'var') || isempty(use_gpu)
        use_gpu=false;
    end
    matcaffe_init(use_gpu,model_def_file, model_file, device_id);
    if ~isfield(opts,'img_mean') || isempty(opts.img_mean)
        d = load('ilsvrc_2012_mean');
        IMAGE_MEAN = d.image_mean;
    else
        IMAGE_MEAN = opts.img_mean;
    end
end

% prepare input
valIdx=find(cellfun(@(x)~isempty(x),bboxes));
bboxes_flat = arrayfun(@(x)[bboxes{x} x*ones(size(bboxes{x},1),1)],valIdx,'uniformoutput',false);
bboxes_flat = cat(1,bboxes_flat{:});
num_bboxes = size(bboxes_flat,1);

if isfield(opts,'context') && ~isempty(opts.context) && opts.context>0
    imgsize=cellfun(@(x)size(x),frames,'uniformoutput',false);
    imgsize=cat(1,imgsize{:});
    frameIdx=bboxes_flat(:,end);
    exec_bboxes_flat=bboxes_flat;
    tmp_bboxes_flat=bboxes_flat(:,1:4);
    tmp_bboxes_flat(:,1)=tmp_bboxes_flat(:,1)-opts.context;
    tmp_bboxes_flat(:,2)=tmp_bboxes_flat(:,2)-opts.context;
    tmp_bboxes_flat(:,3)=tmp_bboxes_flat(:,3)+opts.context;
    tmp_bboxes_flat(:,4)=tmp_bboxes_flat(:,4)+opts.context;
    exec_bboxes_flat(:,1:4)=clip_to_image_group(round(tmp_bboxes_flat),[ones(size(bboxes_flat,1),2) imgsize(frameIdx,2) imgsize(frameIdx,1)]);
else
    exec_bboxes_flat=bboxes_flat;
end


num_batches = ceil(num_bboxes/batch_size);
initic=tic;
for bb = 1:num_batches
    batchtic = tic;
    range = 1+batch_size*(bb-1):min(num_bboxes,batch_size * bb);
    tic
    input_data = prepare_batch(frames,exec_bboxes_flat(range,:),IMAGE_MEAN,batch_size,opts);
    toc, tic
    fprintf('Batch %d out of %d %.2f%% Complete ETA %.2f seconds\n',...
        bb,num_batches,bb/num_batches*100,toc(initic)/bb*(num_batches-bb));
    output_data = caffe('forward', {input_data});
    output_data=output_data{1};
    % change back the width and height to match matlab and caffe
    output_data=permute(output_data,[2 1 3 4]);
    toc
    
    if opts.isvectorize
        output_data = squeeze(output_data);
        output_data = reshape(output_data,[],size(output_data,ndims(output_data)));
        if bb==1
            dim = size(output_data,1);
            cnnFeat = zeros(dim,num_bboxes,'single');
        end
        cnnFeat(:,range) = output_data(:,mod(range-1,batch_size)+1);
    else
        if bb==1
            dim=size(output_data);
            dim=dim(1:end-1);
            assert(length(dim)==3);
            cnnFeat=zeros([dim,num_bboxes],'single');
        end
        cnnFeat(:,:,:,range) = output_data(:,:,:,mod(range-1,batch_size)+1);
    end
    toc(batchtic)
end
toc(initic);




function images = prepare_batch(frames,bboxes,IMAGE_MEAN,batch_size,opts)
% ------------------------------------------------------------------------
if nargin < 2
    d = load('ilsvrc_2012_mean');
    IMAGE_MEAN = d.image_mean; 
end
num_bboxes = size(bboxes,1);
if nargin < 3
    batch_size = num_bboxes;
end

if ~isfield(opts,'image_dim') || isempty(opts.image_dim)
    opts.image_dim=227;
end

IMAGE_DIM = opts.image_dim;
IMAGE_MEAN_RESIZE = imresize(IMAGE_MEAN, [IMAGE_DIM IMAGE_DIM], 'bilinear');

images = zeros(IMAGE_DIM,IMAGE_DIM,3,batch_size,'single');

imgidx=bboxes(:,end);
parfor i=1:num_bboxes
    try
        im = frames{imgidx(i)};
        if max(vec(im))<=1.0
            im=round(im*255);
        end
        bbox=bboxes(i,:);
        im = im(bbox(2):bbox(4),bbox(1):bbox(3),:);
        % resize to fixed input size
        im = single(im);
        im = imresize(im, [IMAGE_DIM IMAGE_DIM], 'bilinear');
        % Transform GRAY to RGB
        if size(im,3) == 1
            im = cat(3,im,im,im);
        end
        % debug
        %imshow(uint8(im));
        % permute from RGB to BGR (IMAGE_MEAN is already BGR)
        im = im(:,:,[3 2 1]) - IMAGE_MEAN_RESIZE;
        % Crop the center of the image
        images(:,:,:,i) = permute(im,[2 1 3]);
    catch
        warning('Problems with file',frames{bboxes(i,end)});
    end
end
