function [cnnFeat_gpu bboxes_flat_gpu] = matcaffe_bbox_multGPU(frames,bboxes,model_def_file,model_file,opts)


% switching between parfor and dswork for parallezation code
% global ds;
% if ~dsmapredisopen
%     launchWrapper('local',2,1);
% end

if ~exist('model_def_file','var') || isempty(model_def_file)
    model_def_file = '/home/fanyix/code/caffe-master/models/bvlc_reference_caffenet/fc7.prototxt';
end
if ~exist('model_file','var') || isempty(model_file)
    model_file = '/home/fanyix/code/caffe-master/models/bvlc_reference_caffenet/bvlc_reference_caffenet.caffemodel';
end

% get number of gpus, usually one of them is used for display
gpu_n=gpuDeviceCount()-1;
if gpu_n<=0
    error('no gpu available to carry out computation, switch to matcaffe_bbox please');
end

if matlabpool('size')==0
  matlabpool('open',gpu_n);
end


if ~exist('opts','var')
    opts=[];
end

if ~isfield(opts,'batch_num') || isempty(opts.batch_num)
    opts.batch_num=gpu_n;
end

if ~isfield(opts,'isnormalize') || isempty(opts.isnormalize)
    opts.isnormalize=true;
end

if ~isfield(opts,'isvectorize') || isempty(opts.isvectorize)
    opts.isvectorize=true;
end

% distribute the tasks
batch_num=opts.batch_num;
n_bboxes_per_frame=cellfun(@(x)size(x,1),bboxes);
n_bboxes_sum=sum(n_bboxes_per_frame);
n_bboxes_accusum=cumsum(n_bboxes_per_frame)/n_bboxes_sum;
gap=1/batch_num;
landmark=0:gap:1;
landmark(1)=[];
s=1;
idxset=cell(batch_num,1);
for i = 1:batch_num
    e=find(n_bboxes_accusum>=landmark(i));
    e=e(1);
    idxset{i}=s:e;
    s=e+1;
end
% take any left one to the last gpu
if s<=length(bboxes)
    idxset{batch_num}=[idxset{batch_num} s:n_bboxes_sum];
end

% carry out the computation
% prepare data
frame_gpu=cell(batch_num,1);
bboxes_gpu=cell(batch_num,1);
cnnFeat_gpu=cell(batch_num,1);
bboxes_flat_gpu=cell(batch_num,1);
for batch_idx=1:batch_num
    frame_gpu{batch_idx}=frames(idxset{batch_idx});
    bboxes_gpu{batch_idx}=bboxes(idxset{batch_idx});
end

% switching between parfor and dswork for parallezation code
% dsupWrap('ds.input.frame_gpu',frame_gpu);
% dsupWrap('ds.input.bboxes_gpu',bboxes_gpu);
% dsupWrap('ds.input.idxset',idxset);
% dsupWrap('ds.input.model_def_file',model_def_file);
% dsupWrap('ds.input.model_file',model_file);
% dsrundistributed('matcaffe_bbox_multGPU_atom(dsidx);',gpu_n);
% % manufacture the results
% fc7_gpu=ds.output.fc7_gpu{:};
% bboxes_flat_gpu=ds.output.bboxes_flat_gpu{:};
% we assume you have more cpu cores than number of gpus
isnormalize=opts.isnormalize;
isvectorize=opts.isvectorize;

% TODO
% parfor (batch_idx=1:batch_num,gpu_n)
for (batch_idx=1:batch_num)
    t=getCurrentTask();
    if ~isempty(t)
        t=t.ID;
        fprintf('lab: %g\n',t);
        assert(t>0 && t<=gpu_n);
    end
    %gpuDevice(gpu_idx);
    % change to matcaffe_bbox_norm if using fc7 feature
    if isnormalize
        [cnnFeat_gpu{batch_idx} bboxes_flat_gpu{batch_idx}]=matcaffe_bbox_norm(frame_gpu{batch_idx},bboxes_gpu{batch_idx},true,model_def_file,model_file,t-1,opts);
    else
        [cnnFeat_gpu{batch_idx} bboxes_flat_gpu{batch_idx}]=matcaffe_bbox(frame_gpu{batch_idx},bboxes_gpu{batch_idx},true,model_def_file,model_file,t-1,opts);
    end
    bboxes_flat_gpu{batch_idx}(:,end)=idxset{batch_idx}(bboxes_flat_gpu{batch_idx}(:,end));
end
if isvectorize
    cnnFeat_gpu=cat(2,cnnFeat_gpu{:});
else
    cnnFeat_gpu=cat(4,cnnFeat_gpu{:});
end
bboxes_flat_gpu=cat(1,bboxes_flat_gpu{:});


% switching between parfor and dswork for parallezation code
% slaveOutdir=ds.sys.slaveOutdir;
% dsdelete('ds.input');
% dsdelete('ds.output');
% if dsmapredisopen
%     dsmapredclose(true);
% end
% clearvars -global
% unix(sprintf('rm -rf %s',slaveOutdir));

