% LRAMain: localized relative attributes
myAddPath('..');
global ds;
whereToRun='local';
switch whereToRun
    case 'local'
        entryDir='/home/LRA_release/data';
    case 'EC2'
        entryDir='/home/data/';
        launchWrapper('EC2',40,10,struct('core',2));
end

dataset_name='LFW10';
ds.store.datasetDir=fullfile(entryDir,dataset_name);
ds.store.imDir=fullfile(ds.store.datasetDir,'data');
ds.store.expDir=fullfile(ds.store.datasetDir,'exp');
ds.store.bufferDir=fullfile(ds.store.expDir,'buffer');
ds.store.printDir=fullfile(ds.store.datasetDir,'printout');
ds.store.pollingDir=fullfile(ds.store.expDir,'polling');
ds.store.LRADetModelDir=fullfile(ds.store.expDir,'LRADetModel');
annotationFilename=fullfile(ds.store.datasetDir,'annotation','annotation.mat');
ds.store.annotation=load(annotationFilename);
map=load('green_colormap.mat');
map=map.map;
ds.store.colormap=map;
relAttr=ds.store.annotation.relAttr;
imNames=ds.store.annotation.im_names;
dsdelete('ds.store.annotation.im_names'); % to speed dssave up
attrNames=ds.store.annotation.attribute_names;


% parameters
ds.store.esvmDefaultParams=esvm_get_default_params;
ds.store.templateW=60;% the width of our detector, below is the height
ds.store.templateH=60;
ds.store.C_O=1000;
ds.store.C_S=1000;
ds.store.spry=[1 2];%spatial pyramid used for appearance matching, assume that spry starts with "1" which means that the first atomN column
ds.store.hogStepSize=8;
perturbRatio=1; % perturbation 
perturbTolerance=0.4;
perturbN=40;
usePascal = true;

% make an IMDB
IMDBFilename=fullfile(ds.store.expDir,'IMDB.mat');
if ~exist(IMDBFilename,'file')
    record=[];
    IMDB=[];
    IMDB.root=ds.store.imDir;
    for attrIdx = 1:length(imNames)
        record(end+1).imgIdx=attrIdx;
        record(end).origImgIdx=attrIdx;
        record(end).file=imNames{attrIdx};
        record(end).flip=0;
        I=imread(fullfile(IMDB.root,record(end).file));
        record(end).size=size(I);
        record(end).gtBbox=[];
        if isfield(ds.store.annotation,'class_labels')
            record(end).gtClass=ds.store.annotation.class_labels(attrIdx);
            record(end).gtClassName=ds.store.annotation.class_names{record(end).gtClass};
        end
        fprintf('IMDB %g/%g created\n',attrIdx,length(imNames));
    end
    IMDB.record=record;
    mySaveFile(IMDBFilename,IMDB);
else
    IMDB=load(IMDBFilename);
    IMDB=IMDB.IMDB;
end
dsupWrap('ds.store.IMDB',IMDB);
clear IMDB;
ds.store.IMDB=processIMDB(ds.store.IMDB,dataset_name);

% learn a dictionary for hog feature cells
dictFilename=fullfile(ds.store.expDir,'DSIFT_dict.mat');
if ~exist(dictFilename,'file')
    dictRandN=1000;
    K=1000;
    gradEngyThresh=500;% this parameter needs tuning among different datasets
    dictRandIdx=myPermute(length(ds.store.IMDB.record),dictRandN);
    dsupWrap('ds.store.operatingIdx',dictRandIdx);
    dsupWrap('ds.store.saveDir',fullfile(ds.store.expDir,'imFeat'));
    dsupWrap('ds.store.isSave',0);
    dsupWrap('ds.store.operatingType','sampling');
    dsrundistributed('extractImFeat(dsidx);',length(ds.store.operatingIdx));
    feat=ds.res.feat;
    dsdelete('ds.res');
    feat=cat(2,feat{:})';
    grad=sqrt(sum(feat.^2,2));
    %[dict,sse,group]=kmeans_bo(feat,K);
    feat(grad<gradEngyThresh,:)=[];
    [dict,sse,group]=kmeans_bo(feat,K-1,100);
    dict=[dict;zeros(1,size(dict,2))];
    mySaveFile(dictFilename,dict);
else
    dict=load(dictFilename);
    dict=dict.dict;
end
ds.store.dict=dict;
ds.store.dictKDtree=vl_kdtreebuild(ds.store.dict');
clear dict;

% get whiten data
getWhitenData;

% print relative attributes
printDir=fullfile(ds.store.printDir,'relativeAttributes');
if ~exist(printDir,'dir')
    for attrIdx = 1:length(attrNames)
        attrName=attrNames{attrIdx};
        subPrintDir=fullfile(printDir,attrName);
        constraints=relAttr(attrIdx==relAttr(:,3),:);
        htmlFilenamePattern=fullfile(subPrintDir,'%g.html');
        for j = 1:size(constraints,1)
            htmlFilename=sprintf(htmlFilenamePattern,j);
            firstIdx=constraints(j,1);
            secIdx=constraints(j,2);
            relation=constraints(j,4);
            switch relation
                case 1
                    relation='stronger';
                case 2
                    relation='weaker';
                case 3
                    relation='equal';
            end
            firstName=imNames{firstIdx};
            secName=imNames{secIdx};
            imgStr={sprintf('attr:<font color="red">%s</font><br>rel:<font color="blue">%s</font><br>%s',attrName,relation,firstName),sprintf('%s',secName)};
            imgList={fullfile(relativepath(ds.store.imDir,subPrintDir),firstName),fullfile(relativepath(ds.store.imDir,subPrintDir),secName)};
            displayPatch(htmlFilename,imgList,imgStr,sprintf('%g.html',j-1),sprintf('%g.html',j+1),1,200,200);
        end
        fprintf('attr: %s finished\n',attrName);
    end
end

% here we get a list of pascal 2007 images that do not contain "person"
pascalNoPersonListFilename=fullfile(ds.store.expDir,'pascalNoPersonList.mat');
if ~exist(pascalNoPersonListFilename,'file')
    pascalDir=fullfile(entryDir,'VOC2007','VOCdevkit','VOC2007','JPEGImages');    
    pascalAnnoFilename=fullfile(ds.store.expDir,'vocAnnotations.mat');
    pascalAnno=load(pascalAnnoFilename);
    pascalAnno=pascalAnno.annotations;
    hasPerson=false(length(pascalAnno),1);
    pascalList=cell(length(pascalAnno),1);
    for ii=1:length(pascalAnno)
        hasPerson(ii)=sum(arrayfun(@(x)strcmp(x.name,'person'),pascalAnno{ii}.object))>0;
        pascalList{ii}=fullfile(pascalDir,pascalAnno{ii}.filename);
    end
    pascalList=pascalList(~hasPerson);
    mySaveFile(pascalNoPersonListFilename,pascalList);
else
    pascalList=load(pascalNoPersonListFilename);
    pascalList=pascalList.pascalList;
end
ds.store.pascalList=pascalList;


% pre-compute the encoding feature
ds.store.imgCodeDir=fullfile(ds.store.expDir,'codes_mirror_fine');
ds.store.integralImgDir=fullfile(ds.store.expDir,'integral_mirror');
ds.store.atomMapDir=fullfile(ds.store.expDir,'atom_mirror');
if ~exist(ds.store.imgCodeDir,'dir')
    dsupWrap('ds.store.operatingIdx',1:length(ds.store.IMDB.record));
    dsupWrap('ds.store.saveDir',ds.store.imgCodeDir);
    dsupWrap('ds.store.operatingType','codes');
    dsupWrap('ds.store.isSave',1);
    dsrundistributed('extractImFeat(dsidx);',length(ds.store.operatingIdx));
end


resBagFilenamePattern=fullfile(ds.store.expDir,'resBag','%s_%g.mat');
sbin=ds.store.esvmDefaultParams.init_params.sbin;
[integralFeat pyrSize scales]=loadMap(ds.store.imgCodeDir,1,sbin);
sizeDiff=sum(abs(bsxfun(@minus,pyrSize,[150 150])),2);
[ignore specIdx]=min(sizeDiff);
specH=pyrSize(specIdx,1);
specW=pyrSize(specIdx,2);
ds.store.universalScales=scales;
ds.store.universalScaledSize=pyrSize;


maxIter=3;
AIdx=1; % split the entire training data into A and B sets, alternate between these two sets
BIdx=2;
for attrIdx=1:length(attrNames)
    attrStartTime=tic;
    allConstraints=relAttr(attrIdx==relAttr(:,3),:);
    allInvolvingImgIdx=unique(vec(allConstraints(:,1:2)));
    allBboxes=arrayfun(@(x)[1 1 x.size(2) x.size(1)],ds.store.IMDB.record(allInvolvingImgIdx),'uniformoutput',false);
    allBboxes=cat(1,allBboxes{:});
    allBboxes=[allBboxes allInvolvingImgIdx specH*ones(size(allBboxes,1),1) specW*ones(size(allBboxes,1),1)];
    printDir=fullfile(ds.store.printDir,'seed',sprintf('%s',attrNames{attrIdx}));
    printDirPattern=fullfile(printDir,'iter%g');
    
    samplingFilename=fullfile(ds.store.bufferDir,sprintf('sampling_%s.mat',attrNames{attrIdx}));
    if ~exist(samplingFilename,'file')
        valConstIdx=find(allConstraints(:,7)==-1);
        valConstraints=allConstraints(valConstIdx,:);
        testConstIdx=find(allConstraints(:,7)==-2);
        testConstraints=allConstraints(testConstIdx,:);
        trConstraints=allConstraints;
        trConstraints([valConstIdx;testConstIdx],:)=[];
        trBboxes=vec(trConstraints(:,1:2));
        trBboxes=allBboxes(ismember(allBboxes(:,5),trBboxes),:);
        valBboxes=vec(valConstraints(:,1:2));
        valBboxes=allBboxes(ismember(allBboxes(:,5),valBboxes),:);
        testBboxes=vec(testConstraints(:,1:2));
        testBboxes=allBboxes(ismember(allBboxes(:,5),testBboxes),:);
        mySaveFile(samplingFilename,trConstraints,valConstraints,testConstraints,trBboxes,valBboxes,testBboxes);
    else
        load(samplingFilename);
    end
    
    
    % sample a large corpus of images
    corpusIdxFilename=fullfile(ds.store.bufferDir,sprintf('corpus_%s.mat',attrNames{attrIdx}));
    if ~exist(corpusIdxFilename,'file')
        % compute an image ranker to globally rank all the images
        printDirPattern=fullfile(printDir,'iter%g');
        globalTrConstraints=trConstraints;
        globalTrBboxes=trBboxes;
        globalValConstraints=valConstraints;
        globalValBboxes=valBboxes;
        [globalW globalDetector globalFeat globalPyrFeat ignore outputBboxes globalAccRes]=getRanker(globalTrConstraints,globalTrBboxes,...
            -1,globalValConstraints,globalValBboxes,ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,...
            ds.store.atomMapDir,ds.store.imgCodeDir,[],[],ds.store.LRADetModelDir,printDirPattern,0,struct('trainDetector',false,'trainRanker',true,'display',false,'mineHard',false,'isJointFeat',false));
        candidateCorpusIdx=1:length(ds.store.IMDB.record);
        candidateCorpusIdx=setdiff(setdiff(candidateCorpusIdx,valBboxes(:,5)),trBboxes(:,5));
        corpusK=1000;
        corpusK=max(corpusK-size(trBboxes,1),0);
        if corpusK>0
            corpusIdx=candidateCorpusIdx(myPermute(length(candidateCorpusIdx),corpusK));
        else
            corpusIdx=[];
        end
        corpusIdx=unique([trBboxes(:,5);vec(corpusIdx)]);
        corpusBboxes=arrayfun(@(x)[1 1 x.size(2) x.size(1)],ds.store.IMDB.record(corpusIdx),'uniformoutput',false);
        corpusBboxes=cat(1,corpusBboxes{:});
        corpusBboxes=[corpusBboxes corpusIdx specH*ones(size(corpusBboxes,1),1) specW*ones(size(corpusBboxes,1),1)];
        [corpusBboxes corpusFeat ignore ignore]=computeFeat(corpusBboxes,ds.store.universalScaledSize,ds.store.universalScales,ds.store.spry,ds.store.IMDB,ds.store.imgCodeDir);
        corpusResp=globalW'*corpusFeat;
        [sortval sortidx]=sort(corpusResp,'descend');
        corpusBboxes=corpusBboxes(sortidx,:);
        corpusFeat=corpusFeat(:,sortidx);

        isOneBboxIndicator=ismember(corpusBboxes(:,5),vec(trConstraints(trConstraints(:,7)==1,1:2)));
        isZeroBboxIndicator=ismember(corpusBboxes(:,5),vec(trConstraints(trConstraints(:,7)==0,1:2)));
        isNotInvolvedBboxIndicator=~isOneBboxIndicator&~isZeroBboxIndicator;
        isOneBboxIdx=find(isOneBboxIndicator);
        isZeroBboxIdx=find(isZeroBboxIndicator);
        isNotInvolvedBboxIdx=find(isNotInvolvedBboxIndicator);
        sourceIndicator=zeros(size(corpusBboxes,1),1);
        sourceIndicator(isOneBboxIdx)=1;
        sourceIndicator(isZeroBboxIdx)=0;
        if ~isempty(isNotInvolvedBboxIdx)
            isNotInvolvedBboxIdxA=isNotInvolvedBboxIdx(myPermute(length(isNotInvolvedBboxIdx),round(length(isNotInvolvedBboxIdx)/2)));
            isNotInvolvedBboxIdxB=setdiff(isNotInvolvedBboxIdx,isNotInvolvedBboxIdxA);
            sourceIndicator(isNotInvolvedBboxIdxA)=-AIdx;
            sourceIndicator(isNotInvolvedBboxIdxB)=-BIdx;
        end
        initK=5;
        [ranking initInferenceIndicator]=selectInitIdx(initK,corpusBboxes,sourceIndicator);
        mySaveFile(corpusIdxFilename,globalW,ranking,corpusBboxes,initInferenceIndicator,sourceIndicator,globalAccRes);
        fprintf('LRAMain: (corpus init) corpus inited for attribute %s, DSIFT global: %g/%g, \n',attrNames{attrIdx},globalAccRes.trAcc,globalAccRes.cvAcc);
    else
        load(corpusIdxFilename);
    end
    
    w=randn(size(ds.store.dict,1)*sum(ds.store.spry.^2),1);
    structure=conStruct([],[],[],[],'specified',struct('specifiedRanking',ranking));
    constraintsA=trConstraints(trConstraints(:,7)==1,:);
    constraintsB=trConstraints(trConstraints(:,7)==0,:);

    c_appr=1;
    c_context=1/20;
    lambda=0.01;
    c_proj=0;
    c_resp=0;
    extRate=0.6;
    initBboxesCandFilename=fullfile(ds.store.bufferDir,sprintf('initBboxesCand_%s.mat',attrNames{attrIdx}));
    if ~exist(initBboxesCandFilename,'file')
        [ignore bboxesCand]=inferenceParallel(structure,[ds.store.templateH ds.store.templateW],w,[],lambda,c_appr,c_proj,c_resp,c_context,extRate,...
           ds.store.spry,[],ds.store.hogStepSize,attrIdx,ds.store.imgCodeDir,ds.store.IMDB,ds.store.imDir,sprintf(printDirPattern,1),ds.store.pollingDir,1);
        mySaveFile(initBboxesCandFilename,bboxesCand);
    else
        load(initBboxesCandFilename);
    end
    
    % iteration params
    c_appr=1.6;
    c_context=0;
    c_resp=0.08;
    
    
    
    for bi=1:length(bboxesCand)
        randInitStartTime=tic;
        printDir=fullfile(ds.store.printDir,sprintf('proc_%g',bi),sprintf('%s',attrNames{attrIdx}));
        resBagFilename=sprintf(resBagFilenamePattern,attrNames{attrIdx},bi);
        rec=cell(maxIter+1,1);
        notinvolvedABIndicator_bi=sourceIndicator;
        notinvolvedABIndicator_bi(notinvolvedABIndicator_bi==1|notinvolvedABIndicator_bi==-AIdx)=AIdx;
        notinvolvedABIndicator_bi(notinvolvedABIndicator_bi==0|notinvolvedABIndicator_bi==-BIdx)=BIdx;
        bboxes=cell(2,1);
        boxedIndicator=cell(2,1);
        bboxes{AIdx}=corpusBboxes(notinvolvedABIndicator_bi==AIdx,:);
        bboxes{BIdx}=corpusBboxes(notinvolvedABIndicator_bi==BIdx,:);
        [a b]=ismember(bboxesCand{bi}(:,5),bboxes{AIdx}(:,5));
        a=find(a);
        b=b(a);
        bboxes{AIdx}(b,:)=bboxesCand{bi}(a,:);
        bboxes{AIdx}(:,6)=bboxesCand{bi}(a(1),6);
        bboxes{AIdx}(:,7)=bboxesCand{bi}(a(1),7);
        boxedIndicator{AIdx}=false(size(bboxes{AIdx},1),1);
        boxedIndicator{AIdx}(b)=true;
        [a b]=ismember(bboxesCand{bi}(:,5),bboxes{BIdx}(:,5));
        a=find(a);
        b=b(a);
        bboxes{BIdx}(b,:)=bboxesCand{bi}(a,:);
        bboxes{BIdx}(:,6)=bboxesCand{bi}(a(1),6);
        bboxes{BIdx}(:,7)=bboxesCand{bi}(a(1),7);
        boxedIndicator{BIdx}=false(size(bboxes{BIdx},1),1);
        boxedIndicator{BIdx}(b)=true;
        origRankingA=bboxes{AIdx}(:,5);
        origRankingB=bboxes{BIdx}(:,5);
        exploreK=zeros(2,1);
        proceedK=zeros(2,1);
        exploreK(AIdx)=ceil(size(bboxes{AIdx},1)/maxIter);
        proceedK(AIdx)=ceil(size(bboxes{AIdx},1)/(maxIter*4));
        exploreK(BIdx)=ceil(size(bboxes{BIdx},1)/maxIter);
        proceedK(BIdx)=ceil(size(bboxes{BIdx},1)/(maxIter*4));
        
        
        detectorRankerFilename=fullfile(ds.store.bufferDir,sprintf('detectorRanker_%s_%g.mat',attrNames{attrIdx},bi));
        if ~exist(detectorRankerFilename,'file')
            procConstraints=cell(2,1);
            procConstraints{AIdx}=constraintsA;
            procConstraints{BIdx}=constraintsB;
            procConstraints{BIdx}=verifyConstraints(procConstraints{BIdx},bboxes{BIdx});
            
            cleanedDetectorFilename=fullfile(ds.store.bufferDir,sprintf('cleanedDetectors_%s_%g.mat',attrNames{attrIdx},bi));
            if ~exist(cleanedDetectorFilename,'file')
                w=cell(2,1);
                detector=cell(2,1);
                feat=cell(2,1);
                pyrFeat=cell(2,1);
                accRes=cell(2,1);

                % Initialize with B
                printDirPattern=fullfile(printDir,'iter%g','B');
                tmpConstraints=verifyConstraints(procConstraints{BIdx},bboxes{BIdx}(boxedIndicator{BIdx},:));
                if usePascal
                    negset = ds.store.pascalList;
                    avoidbboxes = [];
                else
                    negset = getIMDBProperty(bboxes{BIdx}(boxedIndicator{BIdx},5),ds.store.IMDB,'batch_img_path');
                    avoidbboxes = group_into_cell(bboxes{BIdx}(boxedIndicator{BIdx},1:5));
                end
                [w{BIdx} detector{BIdx} outputFeat outputPyrFeat ignore outputBboxes accRes{BIdx}]=getRanker(tmpConstraints,bboxes{BIdx}(boxedIndicator{BIdx},:),bboxes{BIdx}(boxedIndicator{BIdx},5),...
                    valConstraints,[],ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,ds.store.atomMapDir,ds.store.imgCodeDir,negset,avoidbboxes,...
                    ds.store.LRADetModelDir,printDirPattern,0,struct('trainDetector',true,'trainRanker',false,'display',false,'mineHard',false,'isJointFeat',false));
                [a b]=ismember(outputBboxes(:,5),bboxes{BIdx}(:,5));
                a=find(a);
                b=b(a);
                bboxes{BIdx}(b,:)=outputBboxes(a,:);
                if ~isempty(outputPyrFeat)
                    pyrFeat{BIdx}(:,b)=outputPyrFeat(:,a);
                end
                if ~isempty(outputFeat)
                    feat{BIdx}(:,b)=outputFeat(:,a);
                end
                cleanDet;
                detector{AIdx}=detector{BIdx};
                mySaveFile(cleanedDetectorFilename,detector,w,bboxes,feat,pyrFeat);
            else
                load(cleanedDetectorFilename);
            end

            coverIndicator=cell(2,1);
            coverIndicator{AIdx}=false(size(bboxes{AIdx},1),1);
            coverIndicator{BIdx}=false(size(bboxes{BIdx},1),1);
            activeidx=cell(2,1);

            for iter=1:maxIter
                % --------------------------------------------------------
                % FOR A
                % fire detector back on all images in this set and sort them acoording to their highest responses
                sortPrintDir=fullfile(printDir,sprintf('iter%g',iter),'A');
                uncoveridx=find(~coverIndicator{AIdx});
                uncoveridx=uncoveridx(1:min(exploreK(AIdx),length(uncoveridx)));
                if ~isempty(uncoveridx)
                    [ignore permuteIdx]=sortImagesByDetect(detector{AIdx},bboxes{AIdx}(uncoveridx,5),ds.store.IMDB,sortPrintDir,ds.store.universalScales,ds.store.esvmDefaultParams);
                    expandidx=uncoveridx(permuteIdx(1:min(proceedK(AIdx),length(permuteIdx))));
                    activeidx{AIdx}=[activeidx{AIdx}; expandidx];
                    coverIndicator{AIdx}(uncoveridx)=true;
                end
                operRankingA=origRankingA(ismember(origRankingA,bboxes{AIdx}(activeidx{AIdx},5)));
                structureA=conStruct([],[],[],[],'specified',struct('specifiedRanking',operRankingA));

                % extract chain
                printDirPattern=fullfile(printDir,'iter%g','A');
                [ignore operBboxes]=inferenceParallel(structureA,[bboxes{AIdx}(1,6) bboxes{AIdx}(1,7)],w{BIdx},detector{BIdx},lambda,c_appr,c_proj,c_resp,c_context,extRate,...
                    ds.store.spry,[],ds.store.hogStepSize,attrIdx,ds.store.imgCodeDir,ds.store.IMDB,ds.store.imDir,sprintf(printDirPattern,iter),ds.store.pollingDir,0);
                
                % make sure constraints match bboxes
                operConstraints=verifyConstraints(procConstraints{AIdx},operBboxes);
                if isempty(operConstraints)
                    % this is just to make getRanker function happy, anyway we do
                    % not really train ranker here
                    operConstraints=fakeConstraints(operBboxes,attrIdx);
                end
                % learn w from specified bboxes on A
                printDirPattern=fullfile(printDir,'iter%g','A');
                if usePascal
                    negset = ds.store.pascalList;
                    avoidbboxes = [];
                else
                    negset = getIMDBProperty(operBboxes(:,5),ds.store.IMDB,'batch_img_path');
                    avoidbboxes = group_into_cell(operBboxes);
                end
                [w{AIdx} detector{AIdx} outputFeat outputPyrFeat ignore outputBboxes accRes{AIdx}]=getRanker(operConstraints,operBboxes,[],...
                    valConstraints,[],ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,ds.store.atomMapDir,ds.store.imgCodeDir,negset,avoidbboxes,...
                    ds.store.LRADetModelDir,printDirPattern,iter,struct('trainDetector',true,'trainRanker',false,'display',false,'mineHard',false,'isJointFeat',false));
                [a b]=ismember(outputBboxes(:,5),bboxes{AIdx}(:,5));
                a=find(a);
                b=b(a);
                bboxes{AIdx}(b,:)=outputBboxes(a,:);
                if ~isempty(outputPyrFeat)
                    pyrFeat{AIdx}(:,b)=outputPyrFeat(:,a);
                end
                if ~isempty(outputFeat)
                    feat{AIdx}(:,b)=outputFeat(:,a);
                end
                % --------------------------------------------------------
                % FOR B 
                sortPrintDir=fullfile(printDir,sprintf('iter%g',iter),'B');
                uncoveridx=find(~coverIndicator{BIdx});
                uncoveridx=uncoveridx(1:min(exploreK(BIdx),length(uncoveridx)));
                if ~isempty(uncoveridx)
                    [ignore permuteIdx]=sortImagesByDetect(detector{BIdx},bboxes{BIdx}(uncoveridx,5),ds.store.IMDB,sortPrintDir,ds.store.universalScales,ds.store.esvmDefaultParams);
                    expandidx=uncoveridx(permuteIdx(1:min(proceedK(BIdx),length(permuteIdx))));
                    activeidx{BIdx}=[activeidx{BIdx}; expandidx];
                    coverIndicator{BIdx}(uncoveridx)=true;
                end
                operRankingB=origRankingB(ismember(origRankingB,bboxes{BIdx}(activeidx{BIdx},5)));
                structureB=conStruct([],[],[],[],'specified',struct('specifiedRanking',operRankingB));
                % extract chain
                printDirPattern=fullfile(printDir,'iter%g','B');
                [ignore operBboxes]=inferenceParallel(structureB,[bboxes{BIdx}(1,6) bboxes{BIdx}(1,7)],w{AIdx},detector{AIdx},lambda,c_appr,c_proj,c_resp,c_context,extRate,...
                   ds.store.spry,[],ds.store.hogStepSize,attrIdx,ds.store.imgCodeDir,ds.store.IMDB,ds.store.imDir,sprintf(printDirPattern,iter),ds.store.pollingDir,0);
                
                % make sure constraints match bboxes
                operConstraints=verifyConstraints(procConstraints{BIdx},operBboxes);
                if isempty(operConstraints)
                    operConstraints=fakeConstraints(operBboxes,attrIdx);
                end
                % learn w from specified bboxes on B
                printDirPattern=fullfile(printDir,'iter%g','B');
                if usePascal
                    negset = ds.store.pascalList;
                    avoidbboxes = [];
                else
                    negset = getIMDBProperty(operBboxes(:,5),ds.store.IMDB,'batch_img_path');
                    avoidbboxes = group_into_cell(operBboxes);
                end
                [w{BIdx} detector{BIdx} outputFeat outputPyrFeat ignore outputBboxes accRes{BIdx}]=getRanker(operConstraints,operBboxes,[],...
                    valConstraints,[],ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,ds.store.atomMapDir,ds.store.imgCodeDir,negset,avoidbboxes,...
                    ds.store.LRADetModelDir,printDirPattern,iter,struct('trainDetector',true,'trainRanker',false,'display',false,'mineHard',false,'isJointFeat',false));
                [a b]=ismember(outputBboxes(:,5),bboxes{BIdx}(:,5));
                a=find(a);
                b=b(a);
                bboxes{BIdx}(b,:)=outputBboxes(a,:);
                if ~isempty(outputPyrFeat)
                    pyrFeat{BIdx}(:,b)=outputPyrFeat(:,a);
                end
                if ~isempty(outputFeat)
                    feat{BIdx}(:,b)=outputFeat(:,a);
                end
            end
            mySaveFile(detectorRankerFilename,bboxes,activeidx,w,detector);
        else
            load(detectorRankerFilename);
        end
        
        
        % apply the detector back
        operRanking=[bboxes{AIdx}(activeidx{AIdx},5);bboxes{BIdx}(activeidx{BIdx},5)];        
        operRanking=corpusBboxes(ismember(corpusBboxes(:,5),operRanking),5);        
        structure=conStruct([],[],[],[],'specified',struct('specifiedRanking',operRanking));
        finalChainFilename=fullfile(ds.store.bufferDir,sprintf('finalChain_%s_%g.mat',attrNames{attrIdx},bi));
        if ~exist(finalChainFilename,'file')
            printDirPattern=fullfile(printDir,'iter%g','finalDetector');
            [ignore operBboxes]=inferenceParallel(structure,[bboxes{AIdx}(1,6) bboxes{AIdx}(1,7)],w{BIdx},detector{BIdx},lambda,c_appr,c_proj,c_resp,c_context,extRate,...
               ds.store.spry,[],ds.store.hogStepSize,attrIdx,ds.store.imgCodeDir,ds.store.IMDB,ds.store.imDir,sprintf(printDirPattern,maxIter+1),ds.store.pollingDir,0);
            mySaveFile(finalChainFilename,operBboxes);
        else
            load(finalChainFilename);
        end
        
        
        hypoFilename=fullfile(ds.store.bufferDir,sprintf('hypoDetectors_%s_%g.mat',attrNames{attrIdx},bi));
        if ~exist(hypoFilename,'file')
            % make sure constraints match bboxes
            operConstraints=verifyConstraints(trConstraints,operBboxes);
            if isempty(operConstraints)
                operConstraints=fakeConstraints(operBboxes,attrIdx);
            end
            % learn detector from specified bboxes
            printDirPattern=fullfile(printDir,'iter%g','finalDetector');
            if usePascal
                negset = ds.store.pascalList;
                avoidbboxes = [];
            else
                negset = getIMDBProperty(operBboxes(:,5),ds.store.IMDB,'batch_img_path');
                avoidbboxes = group_into_cell(operBboxes);
            end
            [ignore detectorFinal outputFeat outputPyrFeat ignore outputBboxes accResFinal]=getRanker(operConstraints,operBboxes,[],...
                valConstraints,[],ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,ds.store.atomMapDir,ds.store.imgCodeDir,negset,avoidbboxes,...
                ds.store.LRADetModelDir,printDirPattern,maxIter+1,struct('trainDetector',true,'trainRanker',false,'display',false,'mineHard',true,'isJointFeat',true,'contextExtRate',0.35));

            % fire back on all images with relative attribute annotations to gather training data for final ranker training                
            sortPrintDir=fullfile(printDir,sprintf('iter%g',maxIter+1),'finalDetector');
            tmpOperBboxes=operBboxes(:,1:5);
            [ignore permuteIdx finalBboxes ignore hogScores]=sortImagesByDetect(detectorFinal,trBboxes(:,5),ds.store.IMDB,sortPrintDir,ds.store.universalScales,ds.store.esvmDefaultParams,...
                struct('padding',ds.store.esvmDefaultParams.detect_pyramid_padding,'isJointFeat',true,'contextExtRate',0.35));

            hogScoreThresh=-0.3;
            finalK=round(size(finalBboxes,1)/5);
            finalIdx=hogScores>=hogScoreThresh;
            finalBboxes=finalBboxes(finalIdx,:);
            finalBboxes(ismember(finalBboxes(:,5),tmpOperBboxes(:,5)),:)=[];
            finalBboxes=[finalBboxes;tmpOperBboxes];
            finalBboxes=[finalBboxes repmat([ds.store.templateH ds.store.templateW],[size(finalBboxes,1) 1])];
            finalBboxes=finalBboxes(ismember(finalBboxes(:,5),trBboxes(:,5)),:);


            % generate bboxes hypothesis by pertubing the current bboxes
            hypoConcepts.detectorFinal=detectorFinal;
            hypoConcepts.perturbations=cell(perturbN+1,1);
            imgSize=arrayfun(@(x)x.size,ds.store.IMDB.record(finalBboxes(:,5)),'uniformoutput',false);
            imgSize=cat(1,imgSize{:});
            [hypoBboxesSet perturbationH perturbationW]=perturbBboxes(finalBboxes,imgSize,perturbRatio,perturbN,perturbTolerance);
            perturbationH=[0;perturbationH];
            perturbationW=[0;perturbationW];
            hypoBboxesSet=[{finalBboxes};hypoBboxesSet];


            for hypoIdx=1:length(hypoBboxesSet)
                startTime=tic;
                hypoBboxes=hypoBboxesSet{hypoIdx};
                printDirPattern=fullfile(printDir,'iter%g',sprintf('finalRanker_%g',hypoIdx));
                econHypoBboxes=hypoBboxes;
                econValConstraints=valConstraints(valConstraints(:,4)~=3,:);
                operConstraints=verifyConstraints(trConstraints,econHypoBboxes);

                if hypoIdx==1
                    [wFinal ignore outputFeat outputPyrFeat ignore outputBboxes accResFinal precomp]=getRanker(operConstraints,econHypoBboxes,econHypoBboxes(1,5),...
                        econValConstraints,[],ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,ds.store.atomMapDir,ds.store.imgCodeDir,[],[],...
                        ds.store.LRADetModelDir,printDirPattern,maxIter+1,struct('trainDetector',false,'trainRanker',true,'display',false,'saveDisplayInfo',true,'isJointFeat',true,...
                        'contextExtRate',0.35),struct('detector',detectorFinal));
                    valBboxes=precomp.valprecomp.valBboxes;
                    operValBboxes=valBboxes;
                    displayBag=precomp.display;
                    clear precomp;
                else
                    operValBboxes=perturbBboxesVar(valBboxes,perturbationH(hypoIdx),perturbationW(hypoIdx));                    
                    operValConstraints=verifyConstraints(econValConstraints,operValBboxes);
                    [wFinal ignore outputFeat outputPyrFeat ignore outputBboxes accResFinal precomp]=getRanker(operConstraints,econHypoBboxes,econHypoBboxes(1,5),...
                        operValConstraints,operValBboxes,ds.store.IMDB,ds.store.colormap,sbin,ds.store.universalScaledSize,ds.store.universalScales,ds.store.integralImgDir,ds.store.atomMapDir,ds.store.imgCodeDir,[],[],...
                        ds.store.LRADetModelDir,printDirPattern,maxIter+1,struct('trainDetector',false,'trainRanker',true,'display',false,'saveDisplayInfo',true,'isJointFeat',true,...
                        'contextExtRate',0.35),struct('detector',detectorFinal));
                    displayBag=precomp.display;
                    clear precomp;
                end

                hypoConcepts.perturbations{hypoIdx}.wFinal=wFinal;
                hypoConcepts.perturbations{hypoIdx}.displayBag=displayBag;
                hypoConcepts.perturbations{hypoIdx}.perturbationH=perturbationH(hypoIdx);
                hypoConcepts.perturbations{hypoIdx}.perturbationW=perturbationW(hypoIdx);
                hypoConcepts.perturbations{hypoIdx}.hypoBboxes=hypoBboxes;
                hypoConcepts.perturbations{hypoIdx}.valBboxes=operValBboxes;
                hypoConcepts.perturbations{hypoIdx}.accResFinal=accResFinal;
                timeSpent=toc(startTime);
                fprintf('LRAMain: (pert) %s, %g th rand init, %g th perturbation computed, trAcc:%g, cvAcc:%g, time:%g\n',attrNames{attrIdx},bi,hypoIdx,...
                    hypoConcepts.perturbations{hypoIdx}.accResFinal.trAcc,hypoConcepts.perturbations{hypoIdx}.accResFinal.cvAcc,timeSpent);
            end

            % apply the ranker back to operBboxes
            [finalChainBboxes finalChainFeat]=computeFeat(operBboxes,ds.store.universalScaledSize,ds.store.universalScales,ds.store.spry,ds.store.IMDB,ds.store.imgCodeDir);
            finalChainRanker=hypoConcepts.perturbations{1}.wFinal;
            finalChainRankerResp=finalChainRanker'*finalChainFeat;
            hypoConcepts.info.finalChainBboxes=finalChainBboxes;
            hypoConcepts.info.finalChainRankerResp=finalChainRankerResp;
            mySaveFile(hypoFilename,hypoConcepts);

            % sort and print top K sets
            cvAcc=cellfun(@(x)x.accResFinal.cvAcc,hypoConcepts.perturbations(2:end));
            [sortedCvAcc sortedCvAccIdx]=sort(cvAcc,'descend');
            sortedCvAccIdx=sortedCvAccIdx+1;
            dispK=min(1,length(sortedCvAccIdx));
            dispIdx=[1;sortedCvAccIdx(1:dispK)];
            for kk = 1:length(dispIdx)
                hypoIdx=dispIdx(kk);
                printByBag(hypoConcepts.perturbations{hypoIdx}.displayBag);
            end
        end
        randInitTimeSpent=toc(randInitStartTime);
        fprintf('LRAMain: (randinit) %s, %g th rand init, time:%g\n',attrNames{attrIdx},bi,randInitTimeSpent);
    end
    
    
    % ----------------------------------------------------
    % testing phase
    accTableFilename=fullfile(ds.store.bufferDir,sprintf('accTableFilename_%s.mat',attrNames{attrIdx}));
    if ~exist(accTableFilename,'file')
        accTable=zeros(length(bboxesCand)*(perturbN+1),5);
        hypoBboxesBag=cell(length(bboxesCand)*(perturbN+1),1);
        counter=1;
        universalTestIdx=testBboxes(:,5);
        universalTrainIdx=[];
        for bi=1:1:length(bboxesCand)
            hypoFilename=fullfile(ds.store.bufferDir,sprintf('hypoDetectors_%s_%g.mat',attrNames{attrIdx},bi));
            hypoConcepts=load(hypoFilename);
            hypoConcepts=hypoConcepts.hypoConcepts;
            for kk = 1:length(hypoConcepts.perturbations)
                cvAcc=hypoConcepts.perturbations{kk}.accResFinal.cvAcc;
                trAcc=hypoConcepts.perturbations{kk}.accResFinal.trAcc;
                C=hypoConcepts.perturbations{kk}.accResFinal.C;
                accTable(counter,:)=[bi kk trAcc cvAcc C];
                hypoBboxesBag{counter}=hypoConcepts.perturbations{kk}.hypoBboxes;
                counter=counter+1;
            end
            universalTrainIdx=unique([universalTrainIdx;hypoConcepts.perturbations{1}.hypoBboxes(:,5)]);
            fprintf('LRAMain: (acc scan) %g/%g th rand init scanned\n',bi,length(bboxesCand));
        end
        mySaveFile(accTableFilename,universalTrainIdx,universalTestIdx,accTable,hypoBboxesBag);
    else
        load(accTableFilename);
    end
    
    
    ensembleBagFilename=fullfile(ds.store.bufferDir,sprintf('ensemble_%s.mat',attrNames{attrIdx}));
    if ~exist(ensembleBagFilename,'file')
        ensembleK=20;
        grids=[1 4];
        atomN=size(ds.store.dict,1);
        [wEnsemble ensembleAccRes ensembleAccResGlobal ensembleTable moreRes]=ensembleTest(ensembleK,hypoBboxesBag,accTable,universalTrainIdx,valBboxes(:,5),universalTestIdx,...
            trConstraints,valConstraints,testConstraints,ds.store.universalScales,ds.store.universalScaledSize,atomN,grids,ds.store.spry,ds.store.templateH,ds.store.templateW,specH,specW,...
            ds.store.esvmDefaultParams,ds.store.IMDB,ds.store.imgCodeDir,ds.store.bufferDir,ds.store.printDir,attrNames{attrIdx},dataset_name);
        mySaveFile(ensembleBagFilename,wEnsemble,ensembleAccRes,ensembleAccResGlobal,ensembleTable);
    else
        load(ensembleBagFilename);
    end
    
    % uncomment below if you want to try morphing (attribute editor)
    %morphing(ensembleTable,attrNames{attrIdx},ds.store.printDir,ds.store.IMDB,ds.store.universalScaledSize,ds.store.universalScales,ds.store.bufferDir,ds.store.spry,ds.store.imgCodeDir,dataset_name);

    attrTimeSpent=toc(attrStartTime);
    fprintf('LRAMain: (attr) %s, time:%g\n',attrNames{attrIdx},attrTimeSpent);
end























