/*=================================================================
* dp.cc
* written by Yong Jae Lee
*=================================================================*/

#include <math.h>
#include "mex.h"

#define NEGINF -1000000.0

void mexFunction(int nlhs, mxArray *plhs[], 
				 int nrhs, const mxArray *prhs[])
// rhs (input):
// unary (m x n matrix)
// pairwise (m x m x n matrix) 
//
// lhs (output):
// costMat (m x n matrix)
// indexMat (m x n matrix)
{
        
    /* Check for proper number of input and output arguments */    
    if (nrhs != 2) {
		mexErrMsgTxt("2 input arguments required.");
    } 
    if (nlhs != 2){
		mexErrMsgTxt("2 output arguments required.");
    }
    
	// parameters
    mwSize m = mxGetM(prhs[0]); // get number of patches
    mwSize n = mxGetN(prhs[0]); // get number of images
    // set input pointers
    double* unary = mxGetPr(prhs[0]);
    double* pairwise = mxGetPr(prhs[1]);

	// output
    plhs[0] = mxCreateDoubleMatrix(m, n, mxREAL);
    plhs[1] = mxCreateDoubleMatrix(m, n, mxREAL);	
	// set output pointers
	double* costMat = mxGetPr(plhs[0]);
    double* indexMat = mxGetPr(plhs[1]);
	
    // initialize first column of costMat with unary terms
    for (size_t i=0; i<m; i++) {
        costMat[i] = unary[i];             
	}
    
    double val = 0.0;
    double ind = 0.0;
    double maxVal = NEGINF; // in place of neg infinity
    double thisVal = 0.0;
    double maxInd = 0.0;

    for (size_t t=1; t<n; t++) {
        for (size_t j=0; j<m; j++) {  
            maxVal = NEGINF;
            for (size_t i=0; i<m; i++) {
                thisVal = unary[(t*m)+j] + pairwise[(t*m*m)+(j*m)+i] + costMat[(t-1)*m+i];
                if (thisVal > maxVal) {
                    maxVal = thisVal;
                    maxInd = i;
                }            
            }
            costMat[t*m+j] = maxVal;
            indexMat[t*m+j] = maxInd+1; // since matlab indices start at 1
        }
	}   

}
 
