function [wFinal accRes trCorrect valCorrect]=evaluateDriver(trainIdx,trainFeat,testIdx,testFeat,constraints,testConstraints,C)

constraints=verifyConstraints(constraints,trainIdx);
testConstraints=verifyConstraints(testConstraints,testIdx);

involvingImgIdx=unique(vec(constraints(:,1:2)));
[ignore involvingBboxesIdx]=ismember(involvingImgIdx,trainIdx);
if sum(involvingBboxesIdx==0)>0
    error('There are images in the constraints that are not provided with bounding boxes');
end

[ignore localIdx]=ismember(vec(constraints(:,1:2)),involvingImgIdx);% transform image index into local index
localConstraints=constraints;
localConstraints(:,1:2)=reshape(localIdx,[size(constraints,1) 2]);

% form the constraint structure for learning
imgN=length(involvingImgIdx);

inequality=localConstraints(localConstraints(:,4)<3,:);
O=zeros(size(inequality,1),imgN);
isFirstWin=inequality(:,4)==1;
strongIdx=zeros(size(inequality,1),1);
strongIdx(isFirstWin)=inequality(isFirstWin,1);
strongIdx(~isFirstWin)=inequality(~isFirstWin,2);
strongIdx=sub2ind(size(O), (1:size(inequality,1))', strongIdx);
weakIdx=zeros(size(inequality,1),1);
weakIdx(isFirstWin)=inequality(isFirstWin,2);
weakIdx(~isFirstWin)=inequality(~isFirstWin,1);
weakIdx=sub2ind(size(O), (1:size(inequality,1))', weakIdx);
O(strongIdx)=1;
O(weakIdx)=-1;
coSeq=inequality(:,5).*inequality(:,6);
isTrain=inequality(:,7);

equality=localConstraints(localConstraints(:,4)==3,:);
S=zeros(size(equality,1),imgN);
strongIdx=equality(:,1);
weakIdx=equality(:,2);
strongIdx=sub2ind(size(S), (1:size(equality,1))', strongIdx);
weakIdx=sub2ind(size(S), (1:size(equality,1))', weakIdx);
S(strongIdx)=1;
S(weakIdx)=-1;
csSeq=equality(:,5).*equality(:,6);
O=sparse(O);
S=sparse(S);
isTrain=logical(isTrain);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% learn ranker 
% do cross-validation to determine C_O and C_S
if ~exist('C','var') || isempty(C)
    cArray=10.^(-2:1:3);
    acc=zeros(length(cArray),1);
    valCorrect=cell(length(cArray),1);
    for i = 1:length(cArray)
        w=ranksvm_with_sim_fanyi(trainFeat(:,involvingBboxesIdx)',O,S,cArray(i)*coSeq,cArray(i)*csSeq);
        [acc(i) valCorrect{i}]=judgeStandard(w,testIdx,testFeat,testConstraints);
        fprintf('cross val -- c=%g\n',cArray(i));
    end
    [cvAcc maxIdx]=max(acc);
    C=cArray(maxIdx);
    valCorrect=valCorrect{maxIdx};
    fprintf('cross val -- best c=%g, cv acc=%g\n',C,cvAcc);
    wFinal=ranksvm_with_sim_fanyi(trainFeat(:,involvingBboxesIdx)',O,S,C*coSeq,C*csSeq);
    
    r=wFinal'*trainFeat(:,involvingBboxesIdx);
    [trAcc ignore trCorrect]=judgeSparse(r,O);
    accRes.acc=acc;
else
    wFinal=ranksvm_with_sim_fanyi(trainFeat(:,involvingBboxesIdx)',O,S,C*coSeq,C*csSeq);
    r=wFinal'*trainFeat(:,involvingBboxesIdx);
    [trAcc ignore trCorrect]=judgeSparse(r,O);
    [cvAcc valCorrect]=judgeStandard(wFinal,testIdx,testFeat,testConstraints);
end
accRes.trAcc=trAcc;
accRes.cvAcc=cvAcc;
accRes.C=C;
accRes.trCorrect=trCorrect;
accRes.valCorrect=valCorrect;




