function extractImFeat(dsidx)
global ds;
touch;
if ~dsfield(ds,'store')
    dsload('ds.store');
end

% local params
sbin=ds.store.esvmDefaultParams.init_params.sbin;
maxSWH=ds.store.templateH(1);% maximum sliding window height
maxSWW=ds.store.templateW(1);% maximum sliding window width
% minSWH=ds.store.templateH(end);% minimum sliding window height
% minSWW=ds.store.templateW(end);% minimum sliding window width
minDim=90;
if isfield(ds.store,'cnn')
    cnn=ds.store.cnn;
end
% minAreaPortion=0.15 for LFW10
minAreaPortion=0.0375;%0.02;%0.0375;% this quantity controls that the sliding window of size [height width] should be able to correspond to as small as minAreaPortion 
                                            % fraction of the original image
requiredPixelN=((maxSWH*maxSWW)/minAreaPortion);
if ~dsfield(ds,'store','operatingSamplingN')
    samplingN=50;
else
    samplingN=ds.store.operatingSamplingN;
end

startTime=tic;
featParams=ds.store.esvmDefaultParams;
featParams.detect_add_flip=0;
featParams.detect_pyramid_padding=0;
imgIdx=ds.store.operatingIdx(dsidx);
type=ds.store.operatingType;
isSave=ds.store.isSave;

if isSave
    saveFilename=fullfile(ds.store.saveDir,sprintf('%g.mat',imgIdx));
    if exist(saveFilename,'file')
        fprintf('extractImFeat: file exist, img id=%g\n',imgIdx);
        return;
    end
end

% extracting features
I=readImIMDB(imgIdx,ds.store.IMDB);
metainfo=[];
[featMap scales scaledSize]=pixelwiseSIFTPyramid(I,sbin,requiredPixelN,minDim);


switch type
    case 'sampling'
    feat=[];
    for i = 1:length(featMap)
        featMapSub=featMap{i};
        featMapSub=reshape(featMapSub,[],pixelwiseSIFTPyramid);
        feat=[feat;featMapSub(myPermute(size(featMapSub,1),ceil(samplingN/length(featMap))),:)];
    end
    feat=feat';    
    case 'integral'
        feat=cell(length(featMap),1);
        for i = 1:lengtfeath(featMap)
            % quantize feature map into dictionary words
            featStack=reshape(featMap{i},[],pixelwiseSIFTPyramid);
            distMat=pdist2(featStack,ds.store.dict);
            [ignore minIdx]=min(distMat,[],2);  
            NDict=size(ds.store.dict,1);
            minIdx=arrayfun(@(x)indicator(minIdx(x),NDict),1:length(minIdx),'uniformoutput',false);
            minIdx=cat(1,minIdx{:});
            feat{i}=reshape(minIdx,[size(featMap{i},1) size(featMap{i},2) NDict]);
            feat{i}=cumsum(feat{i},1);
            feat{i}=cumsum(feat{i},2);
            %feat{i}=padarray(feat{i},[sbin sbin],nan,'post');% generating the map with the same size as the image
        end
    case 'atomMap'
        feat=cell(length(featMap),1);
        for i = 1:length(featMap)
            % quantize feature map into dictionary words
            featStack=reshape(featMap{i},[],pixelwiseSIFTPyramid);
            distMat=pdist2(featStack,ds.store.dict);
            [ignore minIdx]=min(distMat,[],2);  
            feat{i}=reshape(minIdx,[size(featMap{i},1) size(featMap{i},2)]);
            %feat{i}=padarray(feat{i},[sbin sbin],nan,'post');% generating the map with the same size as the image
        end
    case 'globalHist'
        % note that the very last atom in the dictionary is the all-zero one,
        % and we discard them here
        feat=cell(length(featMap),1);
        for i = 1:length(featMap)
            featStack=reshape(featMap{i},[],pixelwiseSIFTPyramid);
            distMat=pdist2(featStack,ds.store.dict);
            [ignore minIdx]=min(distMat,[],2);  
            feat{i}=hist(minIdx,1:size(ds.store.dict,1)-1)';
            feat{i}=feat{i}/sum(feat{i});
            %feat{i}=vl_homkermap(feat{i},3,'kernel','kinters');
        end
    case 'pixelwisehog'
        feat=hogMap;
    case 'hogSampling'
        patchSize=ds.store.operatingSamplingPatchSize;
        perLevelSamplingN=ceil(samplingN/length(hogMap));
        feat=cell(length(hogMap),1);
        for i = 1:length(hogMap)
            map=hogMap{i};
            feat{i}=zeros(patchSize^2*size(map,3),perLevelSamplingN);
            ppp=reshape(1:size(map,1)*size(map,2),[size(map,1) size(map,2)]);
            ppp=vec(ppp(1:(size(ppp,1)-patchSize+1),1:(size(ppp,2)-patchSize+1)));
            ppp=ppp(myPermute(length(ppp),perLevelSamplingN));
            [uu vv]=ind2sub([size(map,1) size(map,2)],ppp);
            for j = 1:length(uu)
                uuEnd=uu(j)+patchSize-1;
                vvEnd=vv(j)+patchSize-1;
                feat{i}(:,j)=vec(map(uu(j):uuEnd,vv(j):vvEnd,:));
            end
        end
        feat=cat(2,feat{:});
    case 'codes'
        feat=cell(length(featMap),1);
        for i = 1:length(featMap)
            % quantize feature map into dictionary words
            featStack=reshape(featMap{i},[],pixelwiseSIFTPyramid);
            code= featpipem.lib.LLCEncode(featStack',ds.store.dict',5,1e-4,ds.store.dictKDtree,500,true); 
            NDict=size(ds.store.dict,1);
            feat{i}=(code');
            %feat{i}=reshape(full(code)',[size(featMap{i},1) size(featMap{i},2) NDict]);
        end
    case 'deep'
        % do nothing
        feat=featMap;
    otherwise
        error('unknow type of feature specified');
end


if isSave
    mySaveFile(saveFilename,feat,scales,scaledSize,metainfo);
else
    ds.res.feat{dsidx}=feat;
end
timeSpent=toc(startTime);
fprintf('extractImFeat:extracting finished,img id=%g,time:%g\n',imgIdx,timeSpent);