function info=plantTree(constraints,K)
constraints(constraints(:,4)==3,:)=[];
attrIdx=unique(constraints(:,3));
if length(attrIdx)>1
    error('we cannot have more than one attribute for processing');
end

localIdx=unique(vec(constraints(:,1:2)));
imgN=length(localIdx);

localConstraints=constraints(:,1:2);
[ignore tmp]=ismember(vec(localConstraints),localIdx);
tmp=reshape(tmp,[size(localConstraints,1),2]);
localConstraints=tmp;
flipIdx=constraints(:,4)==2;
localConstraints(flipIdx,:)=fliplr(localConstraints(flipIdx,:));

connectMap=zeros(imgN);
for j = 1:size(localConstraints,1)
    firstIdx=localConstraints(j,1);
    secIdx=localConstraints(j,2);
    connectMap(firstIdx,secIdx)=1;
    connectMap(secIdx,firstIdx)=-1;
end
undirectedG=double(connectMap~=0);
optUndirectedG=undirectedG;

selectedTree=[];
treeParentIdx=[];
treeConstraints=[];
treeDepth=[];
treeStructures=[];
collectN=0;
while 1 && length(treeConstraints)<K
    startIdx=find(sum(optUndirectedG,2)>0);
    if isempty(startIdx)
        break;
    end
    linkLength=zeros(length(startIdx),1);
    tmpTreeDepth=cell(length(startIdx),1);
    memberIdx=cell(length(startIdx),1);
    pred=cell(length(startIdx),1);
    for i = 1:length(startIdx)
        [memberIdx{i},pred{i},close,tmpTreeDepth{i}]=graphtraverse(sparse(optUndirectedG),startIdx(i),'METHOD','BFS','DIRECTED',false);
        linkLength(i)=length(memberIdx{i});    
    end

    % form trees
    [maxVal maxIdx]=max(linkLength);
    selectedTree{end+1}=memberIdx{maxIdx};
    treeParentIdx{end+1}=pred{maxIdx}(selectedTree{end});
    treeDepth{end+1}=tmpTreeDepth{maxIdx}(selectedTree{end});
    
    % form constraints format
    tmpTreeConstraints=[];
    treeNodes=selectedTree{end};
    treeNodeParents=treeParentIdx{end};
    structure=[];
    structure.root=1;
    for j=length(treeNodes):-1:1
        % let's start from the leave and back to the root since the nodes
        % are traversed in a BFS manner
        childIdx=(treeNodes(j));
        parentIdx=(treeNodeParents(j));
        structure.data{j}.imgIdx=localIdx(childIdx);
        if parentIdx~=0
            tmpTreeConstraints=[tmpTreeConstraints;localIdx(childIdx) localIdx(parentIdx) attrIdx  convertRA(connectMap(childIdx,parentIdx)) 1 1 0];
            structure.data{j}.parent=find(parentIdx==treeNodes);
            structure.data{j}.isParentStronger=connectMap(parentIdx,childIdx);
        else
            structure.data{j}.parent=0;
            structure.data{j}.isParentStronger=0;
        end
        structure.data{j}.nodeIdx=j;
        structure.data{j}.rankerFeat=[];
        structure.data{j}.pyrFeat=[];
    end
    treeStructures{end+1}=structure;
    treeConstraints{end+1}=tmpTreeConstraints;
    collectN=collectN+length(memberIdx{maxIdx});
    % mask out selected nodes
    optUndirectedG(selectedTree{end},:)=0;
    optUndirectedG(:,selectedTree{end})=0;
    
    fprintf('formed %g th tree (%g nodes), in total %g/%g nodes covered\n',length(selectedTree),length(selectedTree{end}),collectN,imgN);
end

info.localIdx=localIdx;
info.treeConstraints=treeConstraints;
info.treeStructures=treeStructures;
info.selectedTree=selectedTree;
info.treeParentIdx=treeParentIdx;
info.treeDepth=treeDepth;

function out=convertRA(in)
% helper function that converts {1,-1} to {1,2}
if in~=1 && in~=-1
    error('input is in an invalid domain');
end
if in==1
    out=1;
else
    out=2;
end













