function [imgIdx permuteIdx bboxes hogFeat hogFeatResp]=sortImagesByDetect(detector,imgIdx,IMDB,printdir,specifiedScales,esvmParams,options)
% fire detector on all images in this set and then sort them acoording to their highest responses
global ds;

if ~exist('options','var')
    options=[];
end
if ~isfield(options,'isJointFeat')
    options.isJointFeat=false;
end
if ~isfield(options,'mineHard')
    options.mineHard=true;
end
if ~isfield(options,'padding')
    options.padding=0;
end
if ~isfield(options,'printFilename')
    options.printFilename='sort';
end
if ~isfield(options,'withIMDB')
    options.withIMDB=true;
end

% apply detector on validation set to get bboxes
imgIdxbak=imgIdx;
% imgFilenames=arrayfun(@(x)x.file,IMDB(imgIdx),'uniformoutput',false)';
imgSizes=getIMDBProperty(imgIdx,IMDB,'size');


esvmParams.detect_save_features=1;
esvmParams.detect_min_scale=0.2;
esvmParams.detect_pyramid_padding=options.padding;
esvmParams.detect_keep_threshold=-inf;
esvmParams.detect_add_flip=false;
esvmParams.specifiedScales=specifiedScales;

% transform into the ESVM format to make it happy
esvmModel.model.w=detector.w;
esvmModel.model.b=detector.b;
if isfield(detector,'contextw')
    esvmModel.model.contextw=detector.contextw;
end
esvmModel.model.hg_size=size(esvmModel.model.w);
esvmModel.model.init_params=esvmParams.init_params;

dsupWrap('ds.store.operatingIMDB',IMDB);
dsupWrap('ds.store.operatingOptions',options);
dsupWrap('ds.store.operatingDetector',esvmModel);
dsupWrap('ds.store.operatingImgSet',imgIdx);
dsupWrap('ds.store.operatingAvoidBboxes',0);
dsupWrap('ds.store.operatingTopK',1);
dsupWrap('ds.store.operatingDetectorParams',esvmParams);
dsupWrap('ds.store.operatingNegOSThresh',0.3);
dsupWrap('ds.store.operatingGetHogLayer',true);
% if dsmapredisopen()
%     NBatch=ceil(length(ds.store.operatingImgSet)/(length(ds.sys.distproc.availslaves)*1));
% else
%     NBatch=20;
% end
NBatch=20;
batchSeq=unSyncDistribute(length(ds.store.operatingImgSet),NBatch);
dsupWrap('ds.store.operatingNBatch',NBatch);
% dssaveAndWait;
dsrundistributed('mineHardNeg(dsidx);',length(batchSeq));
bboxes=cat(1,ds.res.bboxes{:});
hogFeat=cat(1,ds.res.feat{:});
hogLayer=cat(1,ds.res.hogLayer{:})';
keepidx=cellfun(@(x)isnumeric(x),bboxes);
imgIdx=imgIdx(keepidx);
bboxes=bboxes(keepidx);
hogFeat=hogFeat(keepidx);
hogLayer=hogLayer(keepidx);
bboxes=cat(1,bboxes{:});
scales=bboxes(:,8:10);
bboxes=[bboxes(:,1:4) imgIdx];
imgSizes=imgSizes(keepidx,:);

% TODO: we probably do not want to crop here
% bboxes(:,1:4)=clip_to_image_group(round(bboxes(:,1:4)),imgSizes);
bboxes(:,1:4)=round(bboxes(:,1:4));

hogFeat=cat(2,hogFeat{:});
if options.isJointFeat
    hogFeatResp=[detector.w(:);detector.contextw]'*hogFeat-detector.b;
else
    hogFeatResp=detector.w(:)'*hogFeat-detector.b;
end
[hogFeatResp sortidx]=sort(hogFeatResp,'descend');
imgIdx=imgIdx(sortidx);
bboxes=bboxes(sortidx,:);
hogFeat=hogFeat(:,sortidx);
[ignore permuteIdx]=ismember(imgIdx,imgIdxbak);

dsdelete('ds.res');
dsdelete('ds.store.operatingIMDB');
dsdelete('ds.store.operatingOptions');
dsdelete('ds.store.operatingDetector');
dsdelete('ds.store.operatingImgSet');
dsdelete('ds.store.operatingAvoidBboxes');
dsdelete('ds.store.operatingTopK');
dsdelete('ds.store.operatingDetectorParams');
dsdelete('ds.store.operatingNegOSThresh');
dsdelete('ds.store.operatingNBatch');
dsdelete('ds.store.operatingGetHogLayer');

if ~isempty(printdir)
    % visualization section
    printImgAtom(bboxes,IMDB,fullfile(printdir,options.printFilename));
    printPageAtom(bboxes,hogFeatResp,options.printFilename,printdir);
end


















