function [newmodels,new_models_name] = ...
esvm_train_exemplars_LR(models, train_set, LR_params, params)
% Train models with hard negatives mined from train_set
% [models]: a cell array of initialized exemplar models
% [train_set]: a virtual set of images to mine from
% [params]: localization and training parameters

if length(models) == 0
  newmodels = models;
  new_models_name = '';
  return;
end

if length(params.dataset_params.localdir)==0
  CACHE_FILE = 0;
else
  CACHE_FILE = 1;
end

models_name = models{1}.models_name;
new_models_name = [models_name params.training_function()];

cache_dir =  ...
    sprintf('%s/models/',params.dataset_params.localdir);

cache_file = ...
    sprintf('%s/%s.mat',cache_dir,new_models_name);

cache_file_stripped = ...
    sprintf('%s/%s-stripped.mat',cache_dir,new_models_name);

if CACHE_FILE == 1 && fileexists(cache_file_stripped)
  newmodels = load(cache_file_stripped);
  newmodels = newmodels.models;
  return;
end


if CACHE_FILE == 1 && fileexists(cache_file)
  newmodels = load(cache_file);
  newmodels = newmodels.models;
  return;
end

DUMPDIR = sprintf('%s/www/svs/%s/',params.dataset_params.localdir, ...
                  new_models_name);

%display of SV pdfs disabled
%if CACHE_FILE==1 && params.dataset_params.display ==1 && ~exist(DUMPDIR,'dir')
%  mkdir_recursive(DUMPDIR);
%end

final_directory = ...
    sprintf('%s/models/%s/',params.dataset_params.localdir,...
            new_models_name);

%make results directory if needed
if CACHE_FILE == 1 && ~exist(final_directory,'dir')
  mkdir_recursive(final_directory);
end

% % randomize chunk orderings
% if CACHE_FILE == 1
%   myRandomize;
%   ordering = randperm(length(models));
% else
%   ordering = 1:length(models);
% end
% models = models(ordering);


% parameters for updating factors
update_rate = 2; % update factors every update_rate iterations
maxPauseTime = 0.05;
mini_batch_msize = 8;
mini_batch_dsize = 200;
global_model_idx = LR_params.global_model_idx;
batch_idx = LR_params.batch_idx;
N_models = LR_params.N_models;
machine_lock_prefix = LR_params.machine_lock_prefix;

if length(models) ~= length(global_model_idx)
    error('length of models and global index not match...\n');
end


allfiles = cell(length(models), 1);
filer2fill = cell(length(models), 1);
filer2final = cell(length(models), 1);



% LR_data contains field 'UD' and 'factor_path' (which contains the path storing the factor files)
factor_path = LR_params.factor_path;
factor_file = LR_params.factor_file;
response_file_prefix = LR_params.response_file_prefix;
% setting_file = LR_params.setting_file;

keep_going = ones(length(models),1);
iteration = 1;
m = cell(length(models),1);
count = 1; % for synchronizing with parameter server (master)

while sum(keep_going) > 0
    
    
    
    % some data could only be loaded once
    if iteration == 1
%         enter_lock(fullfile(factor_path,setting_file),maxPauseTime);
%         load(fullfile(factor_path,setting_file));
%         exit_lock(fullfile(factor_path,setting_file));
        
        complete_model_idx = [global_model_idx global_model_idx + N_models];
%         N_mini_batch_m = ceil(length(models)/mini_batch_msize);
%         N_mini_batch_d = ceil(length(precomputation_list)/mini_batch_dsize);

        
        % manufacturing data structure LR passed into training 
        precomputation_list = cellfun(@(x)fullfile(LR_params.HOG_path,x),LR_params.precomputation_list,...
            'UniformOutput',false);
        LR.precomputation_list = precomputation_list;
        LR.lambda = LR_params.lambda;
        LR.ratio = LR_params.ratio;
        LR.maxIter = LR_params.maxIter;
        LR.tol = LR_params.tol;
    else
        % read info from given path
        enter_lock(fullfile(factor_path,factor_file),maxPauseTime);
        load(fullfile(factor_path,factor_file));
        exit_lock(fullfile(factor_path,factor_file));
        est_R = U*V;
    end
    
   
    % manufacture UDs (use mini batch to reduce times of reading files and thus speed up)
    model_count = 1;
    for i = 1:length(models)
        
        if iteration == 1
            m{i} = models{i};
            
            % the first time entering this loop, do initialization
            m{i}.train_set = train_set;
            
            m{i}.mining_queue = esvm_initialize_mining_queue(m{i}.train_set);

            % Add mining_params, and params.dataset_params to this exemplar
            m{i}.mining_params = params;
            m{i}.dataset_params = params.dataset_params;

            % Append '-svm' to the mode to create the models name
            m{i}.models_name = new_models_name;
            
            [complete_file] = sprintf('%s/%s.mat',final_directory,num2str(m{i}.objectid));
            [basedir, basename, ext] = fileparts(complete_file);
            filer2fill{i} = sprintf('%s/%%s.%s.mat',basedir,basename);
            filer2final{i} = sprintf('%s/%s.mat',basedir,basename);  
            allfiles{i} = filer2final{i};

            LR.isLR = 0;
        else
            
            if mod(model_count,mini_batch_msize) == 1
                
                % read data for a batch of models
                upper = min(i+mini_batch_msize-1,length(models));
                UD_vec = cell(upper-i+1,1);
                flag_matrix = true(2*length(precomputation_list),upper-i+1);
                
                for j = 1:length(precomputation_list)
                    load(precomputation_list{j});
                    for k = i:upper
                        hg_size = models{k}.model.hg_size;

                        uus_orig = UD{j}(k,1);
                        vvs_orig = UD{j}(k,2);
                        level_orig = UD{j}(k,3);
                        valid_flag_orig = UD{j}(k,4);
                        if valid_flag_orig>=0
                            UD_vec{k-i+1}(:,j*2-1) = vec(precomputation.orig.hog{level_orig}(...
                                uus_orig:uus_orig+hg_size(1)-1,vvs_orig:vvs_orig+hg_size(2)-1,:));
                        else
                            UD_vec{k-i+1}(:,j*2-1) = zeros(prod(hg_size),1);
                            flag_matrix(j*2-1,k-i+1) = false;
                        end
                        
                        uus_flip = UD{j}(k+length(models),1);
                        vvs_flip = UD{j}(k+length(models),2);
                        level_flip = UD{j}(k+length(models),3);
                        valid_flag_flip = UD{j}(k+length(models),4);
                        if valid_flag_flip>=0
                            UD_vec{k-i+1}(:,j*2) = vec(precomputation.flip.hog{level_flip}(...
                            uus_flip:uus_flip+hg_size(1)-1,vvs_flip:vvs_flip+hg_size(2)-1,:));
                        else
                            UD_vec{k-i+1}(:,j*2) = zeros(prod(hg_size),1);
                            flag_matrix(j*2,k-i+1) = false;
                        end
                    end
                end
                model_count = 1;
            end
            
            LR.val_idx = flag_matrix(:,model_count);
            LR.isLR = 1;
            LR.UD_vec = UD_vec{model_count};
            
            
            LR.overall_mean = overall_mean;
            LR.model_mean = [model_mean(global_model_idx(i));...
                model_mean(global_model_idx(i) + N_models)];
            LR.task_mean = task_mean;
            LR.r = est_R([global_model_idx(i) global_model_idx(i) + N_models],:);
            model_count = model_count + 1;
        end
        
        m{i}.iteration = iteration;
        filer2 = sprintf(filer2fill{i},num2str(m{i}.iteration));
        
        if keep_going(i) == 0 
            continue;
        end
        
        if ~isfield(m{i},'mining_stats')
          total_mines = 0;
        else
          total_mines = sum(cellfun(@(x)x.total_mines,m{i}.mining_stats));
        end
        m{i}.total_mines = total_mines;
        
        m{i} = esvm_mine_train_iteration_LR(m{i}, LR);
        
        if ((total_mines >= params.train_max_mined_images) || ...
              (isempty(m{i}.mining_queue))) || ...
              (m{i}.iteration == params.train_max_mine_iterations)

          keep_going(i) = 0;      
          %bump up filename to final file
          filer2 = filer2final{i};
        end
        msave = rmfield(m{i},'train_set');

        %Save the current result
        if CACHE_FILE == 1
          savem(filer2,msave);
        end

        %delete old files
        if m{i}.iteration > 1
          for q = 1:m{i}.iteration-1
            filer2old = sprintf(filer2fill{i},num2str(q));
            if fileexists(filer2old)
              if CACHE_FILE == 1
                delete(filer2old);
              end
            end
          end
        end

        if keep_going(i)==0
          fprintf_oct(1,[' ### End of training of ' num2str(i) 'th model ... \n']);
        end
    end
    
    % finish one round on all models, update factors
    
%     w_all = cellfun(@(x)x.model.w,m,'UniformOutput',false);
%     w_all = cat(2,w_all{:});
%     b_all = cellfun(@(x)x.model.b,m);
    
    % need facilities to synchronize 
    if mod(iteration-1,update_rate) == 0
        % compute the new response matrix using new parameter w
%         new_R_block = w_all'*UD - repmat(b_all,[1 size(UD,2)]);

        tic;
        fprintf_oct(['Machine ' num2str(batch_idx) ', iteration ' num2str(iteration) ', updating iteration...\n']);
        [R UD] = get_response_matrix(precomputation_list,m,params,LR_params);
%         for j = 1:length(UD)
%             UD{j}(complete_model_idx,:) = new_UD{j};
%         end
        
        response_filename = [response_file_prefix num2str(batch_idx)];
        save(fullfile(factor_path,response_filename),'R','UD','complete_model_idx');
        
        % write finish lock 
        mkdir_recursive(fullfile(factor_path,[machine_lock_prefix num2str(batch_idx) '.lock']));
        
        
        fprintf_oct(['Machine ' num2str(batch_idx) ', iteration ' num2str(iteration) ...
            ', updating finished, taking time ' num2str(toc) ' ...\n'])
        fprintf_oct(['Machine ' num2str(batch_idx) ' waiting...\n']);
        

        stop_here = 1;
        while stop_here
            % wait for a random period time and see if parameter server has
            % finished updating factors
            stop_time = randi(maxPauseTime*100)/100;
            pause(stop_time);

            if ~exist(fullfile(factor_path,['master_waiting_' num2str(count) '.lock']),'dir')
                stop_here = 0;
            end
            % finished updating, all nodes could get the updated factors now
        end
        count = count + 1;
    end
    
    
    iteration = iteration + 1;
end


% coming here means finish training in this batch, mark the status file
% write final finish lock 
mkdir_recursive(fullfile(factor_path,[machine_lock_prefix num2str(batch_idx) '.finallock']));



[allfiles] = sort(allfiles);
%Load all of the initialized exemplars
CACHE_FILE = 1;
STRIP_FILE = 1;

if new_models_name(1) == '-'
  CACHE_FILE = 0;
  STRIP_FILE = 0;
end

DELETE_INITIAL = 0;
newmodels = esvm_load_models(params.dataset_params, new_models_name, allfiles, ...
                          CACHE_FILE, STRIP_FILE, DELETE_INITIAL);

function savem(filer2,m)
save(filer2,'m');
