% Author: Fanyi Xiao (fanyix@cs.cmu.edu)

function dsmapredAddSlave(startIdx,endIdx,target,outdir,masterDir)
  %general setup; should not need to be modified.
  qsubopts='';
  dsworkpath=mfilename('fullpath');
  dotpos=find(dsworkpath=='/');
  dsworkpath=dsworkpath(1:(dotpos(end)-1));
  % sanity check
  if ~strcmp(outdir(end),'/')
    outdir = [outdir '/'];
  end
  if(strcmp(target,'local'))
    submitlocal=1;
  else
    submitlocal=0;
  end
  distprocdir=[outdir '/ds/sys/distproc/'];
  slaveIdx = startIdx:endIdx;
  currchunk={};
  for(i=1:length(slaveIdx))
     %generate the script.
     idx = slaveIdx(i);
     disp(['submitting job ' num2str(idx)]);
     tmpOutFName = [distprocdir 'qsubfile' num2str(idx) '.sh'];
     fid = fopen(tmpOutFName, 'w');
     %this should point to the matlab binary accessible on worker nodes.
     logfile=[distprocdir 'output' num2str(idx) '.log'];
     % begin writing the script that will be run via qsub. 
     mlpipe=[distprocdir '/mlpipe' num2str(idx)];
     fprintf(fid, '%s\n',['#!/bin/bash'] );
     fprintf(fid, '%s\n',['cd "' masterDir '";'] );
     fprintf(fid, '%s\n',['runtail=1;']);
     fprintf(fid, '%s\n',['if [[ ! -p ' mlpipe ' ]]; then']);
     fprintf(fid, '%s\n',['   if mkfifo "' mlpipe '"; then']);
     fprintf(fid, '%s\n',['     runtail=1']);
     fprintf(fid, '%s\n',['   else']);
     fprintf(fid, '%s\n',['     runtail=0']);
     fprintf(fid, '%s\n',['   fi']);
     fprintf(fid, '%s\n',['fi']);
     % on each worker, matlab is run with the fifo (mlpipe) as the STDIN that it reads commands from; this is the magic
     % that lets dscmd work.  Output is sent to a logfile.  
    if(submitlocal)
        matlabbin='/usr/local/MATLAB/R2012a/bin/matlab';
    else
        %this should point to the matlab binary accessible on worker nodes.
        [~,matlabbin]=unix('which matlab');
        matlabbin=matlabbin(1:end-1);
    end
    sct='-nojvm';
    %the actual command run in matlab once everything is set up.
    matlabcmd=['addpath(''' dsworkpath ''');dsmapreducerwrap(' num2str(idx) ',''' outdir ''',1);']
    runnable = [matlabbin ' -nodesktop -nosplash ' sct ' -r "' matlabcmd '"'];
     if(exist(mlpipe,'file'))
     else
       inputpipe=[];
     end
     fullcmd=[' ' runnable ' 2>&1 >> "' logfile '" &' ];
     fullcmd2=[' ' runnable ' 2>&1 < ' mlpipe ' >> "' logfile '" &' ];
     % note that this sh file is responsiible for dealing with dsmapredrestart
     % (as well as dealing with matlab crashing badly/segfaulting).  This while
     % loop simply runs matlab and, if its return code is nonzero, restart matlab.
     fprintf(fid, '%s\n',['rm ' logfile ]);
     fprintf(fid, '%s\n',['false;']);
     fprintf(fid, '%s\n',['while [ $? -gt 0 ]; do']);
     fprintf(fid, '%s\n',['if [ $runtail ]; then']);
     fprintf(fid, '%s\n',['   ' fullcmd2]);
     fprintf(fid, '%s\n',['   mypid=$!']);
     fprintf(fid, '%s\n',['   echo "a" > ' mlpipe]);
     fprintf(fid, '%s\n',['else']);
     fprintf(fid, '%s\n',['   ' fullcmd]);
     fprintf(fid, '%s\n',['   mypid=$!']);
     fprintf(fid, '%s\n',['fi']);
     %if(~iscompiled)
     fprintf(fid, '%s\n',['echo $mypid > ' distprocdir 'matpid' num2str(idx) ]);
     fprintf(fid, '%s\n',['wait $mypid' ]);
     fprintf(fid, '%s\n',['done' ]);
     fclose(fid);
     unix(['chmod 755 ' tmpOutFName]);
     % actually submit the jobs.  tmoOutFName is the actual script that needs to be run on each node.  In my case,
     % warp.hpc1.cs.cmu.edu was the root node of the cluster, which handled handled queueing for the cluster via
     % torque.
    if(submitlocal)
        unix(['sleep ' num2str(floor(i/2)) ' && ' tmpOutFName ' &']);
    else
        currchunk{1}=tmpOutFName;
        submitchunk(currchunk,qsubopts,idx,target,outdir);
        currchunk=[];
    end
  end
  newComers.newSlaves = startIdx:endIdx;
  save(fullfile(distprocdir,'newComers.mat'),'newComers');
end
function submitchunk(chunk,qsubopts,chunkid,target,outdir)
      distprocdir=[outdir 'ds/sys/distproc/'];
      logfileerr=[distprocdir 'stderr' num2str(chunkid) '.log'];
      logfileout=[distprocdir 'stdout' num2str(chunkid) '.log'];
       logstring = ['-e "' logfileerr '" -o "' logfileout '"']; 
       if(numel(chunk)==1)
         submitScr=chunk{1};
       else
         submitScr=[distprocdir 'qsubchunk' num2str(chunkid) '.sh'];
         fid = fopen(submitScr, 'w');
         for(i=1:numel(chunk))
           fprintf(fid, '%s\n',[chunk{i} ' &'] );
         end
         fprintf(fid, '%s\n','wait;' );
         fclose(fid)
       end
       qsub_cmd=['source /etc/profile; qsub -l nodes=1:ppn=2,mem=4gb -V -N FPS-' ...
           num2str(chunkid) ' ' qsubopts ' ' logstring ' ' submitScr]
       ssh_cmd = sprintf(['ssh ' target ' ''%s'''], qsub_cmd)
       unix(ssh_cmd);
end