function rs1 = esvmQuantDetect(quantizedStruct,lutStruct,models,params)
if length(quantizedStruct)>1
    flipDet = 1;
else
    flipDet = 0;
end
bs = lutStruct.bs;
ws = lutStruct.ws;
modelSizes = cellfun2(@(x)x.model.hg_size,models);
modelSizes = cat(1,modelSizes{:});
params.detect_add_flip = 0;
rs1 = esvmQuantDetectDriver(quantizedStruct{1},lutStruct,ws,bs,modelSizes,params);
rs1 = prune_nms(rs1, params);
params.detect_add_flip = 1;
rs2 = esvmQuantDetectDriver(quantizedStruct{2},lutStruct,ws,bs,modelSizes,params);
rs2 = prune_nms(rs2, params);
for q = 1:length(rs1.bbs)
    rs1.bbs{q} = cat(1,rs1.bbs{q},rs2.bbs{q});
end
if isfield(rs1,'r') && isfield(rs2,'r')
    rs1.r = [rs1.r; rs2.r];
end

function resstruct = esvmQuantDetectDriver(quantizedStruct,lutStruct,ws,bs,...
    modelSizes,params)
% compute responses for the original image
NModels = size(modelSizes,1);
sbin = params.init_params.sbin;
codes = quantizedStruct.codes;
compilationMode = 1;

for i = 1:length(lutStruct.lut)
    lutStruct.lut{i} = double(lutStruct.lut{i});
end

switch params.accelerationOption
    case 1
        fullAOMP = full(quantizedStruct.AOMP);
        
        %tic
        %r = zeros(size(lutStruct.lut{1},1),size(codes,2));
        %for i = 1:length(lutStruct.lut)
        %    lut = lutStruct.lut{i};
        %    resp = lut*quantizedStruct.AOMP(:,codes(i,:));
        %    r = r + resp;
        %end
        %toc
        
        r = fastSparseMultiplication(fullAOMP,codes,...
            lutStruct.lut,params.OMPLambda,4);
        r = bsxfun(@minus,r,bs);
    case 2
        %lut = cat(2,lutStruct.lut{:});
        %tic
        %offsets = repmat([0:1:(size(codes,1)-1)]*size(lutStruct.lut{1},2),[size(codes,2) 1]);
        %lutIdx = quantizedStruct.wordIdx(vec(codes')) + vec(offsets);
        %r2 = lut(:,lutIdx);
        %r2 = reshape(r2,[size(lutStruct.lut{1},1) size(codes,2) length(lutStruct.lut)]);
        %r2 = sum(r2,3);
        %toc
        
        %tic
        %r = zeros(size(lutStruct.lut{1},1),size(codes,2));
        %wordIdx = reshape(quantizedStruct.wordIdx(vec(codes)),size(codes));
        %for i = 1:length(lutStruct.lut)
        %  lut = lutStruct.lut{i};
        %  resp = lut(:,wordIdx(i,:));
        %  r = r + resp;
        %end
        %toc
        
        wordIdx = reshape(quantizedStruct.wordIdx(vec(codes)),size(codes))';
        % fast referencing function
        r = fastReferencing(wordIdx,lutStruct.lut);
        r = bsxfun(@minus,r,bs);
end
if isfield(params,'getResponseMap') && params.getResponseMap
    resstruct.r = r;
end
% re-estimate top model-location pairs
% [sortedRVal,sortedRIdx] = psort(-vec(r),numel(r));
[sortedRVal,sortedRIdx] = psort(-vec(r),...
    round(params.reEstRatio*numel(r)));
[reEstX reEstY] = ind2sub(size(r),double(sortedRIdx));
quanWindowIdx = codes(:,reEstY);
reEstR = evalTemplateFromParts(ws,reEstX,quantizedStruct.X,...
    quanWindowIdx,4);
reEstR = reEstR-bs(reEstX);
if compilationMode == 1 % normal mode
    r = -inf(size(r));
    r(sortedRIdx) = reEstR;
end

% reEstR = zeros(1,length(reEstY));
% for i = 1:size(codes,1)
%     reEstR = reEstR + sum(ws{i}(:,reEstX).*quantizedStruct.X(:,codes(i,reEstY)),1);
% end
% reEstR = reEstR-bs(reEstX)';

uus = quantizedStruct.uus;
vvs = quantizedStruct.vvs;
scales = quantizedStruct.scales;
padder = quantizedStruct.padder;
resstruct.bbs = cell(NModels,1);


if compilationMode == 1 % normal mode
    for exid = 1:NModels
      goods = find(r(exid,:) >= params.detect_keep_threshold);
      if isempty(goods)
        continue    
      end

      [sorted_scores,bb] = ...
          psort(-r(exid,goods)',...
                min(params.detect_max_windows_per_exemplar, ...
                    length(goods)));
      bb = goods(bb);

      sorted_scores = -sorted_scores';

      scalesEx = scales(bb);
      curuus = uus(bb);
      curvvs = vvs(bb);

      o = [curuus' curvvs'] - padder;

      bbs = ([o(:,2) o(:,1) o(:,2)+modelSizes(exid,2) ...
               o(:,1)+modelSizes(exid,1)] - 1) .* ...
                 repmat(sbin./scalesEx',1,4) + 1 + repmat([0 0 -1 ...
                        -1],length(scalesEx),1);

      bbs(:,5:12) = 0;
      bbs(:,5) = (1:size(bbs,1));
      bbs(:,6) = exid;
      bbs(:,8) = scalesEx;
      bbs(:,9) = uus(bb);
      bbs(:,10) = vvs(bb);
      bbs(:,12) = sorted_scores;

      if (params.detect_add_flip == 1)
        bbs = flip_box(bbs,quantizedStruct.imgSize);
        bbs(:,7) = 1;
      end
      resstruct.bbs{exid} = bbs;
    end
elseif compilationMode == 2 % efficient mode
    sortedRVal = -sortedRVal;
    uus = uus(reEstY);
    vvs = vvs(reEstY);
    scales = scales(reEstY);
    for exid = 1:NModels
      bb = reEstX == exid;
      if sum(bb) == 0
        continue    
      end
      sorted_scores = sortedRVal(bb);
      scalesEx = scales(bb);
      curuus = uus(bb);
      curvvs = vvs(bb);
      o = [curuus' curvvs'] - padder;
      bbs = ([o(:,2) o(:,1) o(:,2)+modelSizes(exid,2) ...
               o(:,1)+modelSizes(exid,1)] - 1) .* ...
                 repmat(sbin./scalesEx',1,4) + 1 + repmat([0 0 -1 ...
                        -1],length(scalesEx),1);
      bbs(:,5:12) = 0;
      bbs(:,5) = (1:size(bbs,1));
      bbs(:,6) = exid;
      bbs(:,8) = scalesEx;
      bbs(:,9) = curuus';
      bbs(:,10) = curvvs';
      bbs(:,12) = sorted_scores;

      if (params.detect_add_flip == 1)
        bbs = flip_box(bbs,quantizedStruct.imgSize);
        bbs(:,7) = 1;
      end
      resstruct.bbs{exid} = bbs;
    end
end
resstruct.xs = [];
% fprintf(1,'\n');

function [rs preserved_idx] = prune_nms(rs, params)
%Prune via nms to eliminate redundant detections

%If the field is missing, or it is set to 1, then we don't need to
%process anything.  If it is zero, we also don't do NMS.
preserved_idx = [];
if ~isfield(params,'detect_exemplar_nms_os_threshold') || (params.detect_exemplar_nms_os_threshold >= 1) ...
      || (params.detect_exemplar_nms_os_threshold == 0)
  return;
end

[rs.bbs preserved_idx] = cellfun(@(x)esvm_nms(x,params.detect_exemplar_nms_os_threshold),...
    rs.bbs,'uniformoutput',false);

if ~isempty(rs.xs)
  for i = 1:length(rs.bbs)
    if ~isempty(rs.xs{i})
      %NOTE: the fifth field must contain elements
      rs.xs{i} = rs.xs{i}(:,rs.bbs{i}(:,5) );
    end
  end
end


