function [beta cost_rec] = esvm_learn_sigmoid_lbfgs(scores, os, iter)
% function beta = esvm_learn_sigmoid(scores, os)
% Fit a sigmoid to the scores verus os. Examples with high scores
% and high os's will "fit" well.  This is a soft way of counting
% the number of "good" detections before the first bad one.
%
% Instances with os>.5 are treated as positives, instances with os
% between [.2 and .5] are treated as dont-cares, and os<.2 is treated
% as negative.  The parameters have been tweaked such that raw SVM
% output scores which assign a 1.0 for the positive and scores around
% -.9 for negatives, produce reasonable fits.  If the scores do not
% satisfy this distribution, then there might be a local minimum
% problem -- check this first if fitting is not working for you)
%
% Copyright (C) 2011-12 by Tomasz Malisiewicz
% All rights reserved.
% 
% This file is part of the Exemplar-SVM library and is made
% available under the terms of the MIT license (see COPYING file).
% Project homepage: https://github.com/quantombone/exemplarsvm
% Modified by Fanyi Xiao

x = scores;
y = os;

% hi_thresh = 0.4;
% lo_thresh = 0.2;
% %capping to one, (DO THIS OR NOT? YES)
% y(y>=hi_thresh) = 1;
% y(y<=lo_thresh) = 0;
% bads = y>lo_thresh & y<hi_thresh;
% y(bads) = [];
% x(bads) = [];

reg_constant = .000001;
cost_rec = zeros(iter,1);

guess2 = 100;
if sum(y>.5) > 0
    guess2 = mean(x(y>.5));
    beta = [3.0; guess2];
else
    beta = [3.0; guess2];
    return;
end

%  Use minFunc to minimize the function
options.Method = 'lbfgs'; 
options.display = 'off';
curcost = inf;
curbeta = beta;
for i = 1:iter
    % check gradient
    %[J grad] = obj_fun(beta,reg_constant,x,y);
    %numgrad = computeNumericalGradient( @(beta) obj_fun(beta,reg_constant,x,y), beta);
    %disp([numgrad grad]); 

    [beta_i, cost_i] = minFunc( @(beta) obj_fun(beta,reg_constant,x,y), beta, options);
    cost_rec(i) = cost_i;
    if cost_i<curcost
        curbeta = beta_i;
        curcost = cost_i;
    end
    
    beta1 = 10*rand(1);
    beta2 = -rand(1);
    beta = [beta1; beta2];
    % fprintf([num2str(i) 'th iter finished. cost: ' num2str(cost_i) '\n']);
end
beta = curbeta;

function [J grad] = obj_fun(beta,reg_constant,x,y)
e = exp(-beta(1)*(x-beta(2)));
J = mean((1./(1+e)-y).^2) + reg_constant*beta(1).^2;
grad = [2*(mean(((x-beta(2)).*e)./((1+e).^2).*(1./(1+e)-y))+reg_constant*beta(1)); 2*mean((1./(1+e)-y).*(-beta(1)./((1+e).^2).*e))];








