function [resstruct,feat_pyramid] = esvm_recDetectFast(I, models, params)

if isempty(models)
  fprintf(1,'Warning: empty models in esvm_detect\n');
  resstruct.bbs{1} = zeros(0,0);
  resstruct.xs{1} = zeros(0,0);
  feat_pyramid = [];
  return;
end

if ~iscell(models)
  models = {models};
end

if isfield(models{1},'mining_params') && ~exist('params','var')
  params = models{1}.mining_params;
elseif ~exist('params','var')
  params = esvm_get_default_params;
end

if ~isfield(params,'nnmode')
 params.nnmode = '';
end

if ~isfield(params,'save_fired_HOG_loc')
    params.save_fired_HOG_loc = 0;
end

doflip = params.detect_add_flip;

params.detect_add_flip = 0;

[rs1, t1] = esvm_detectdriver(I, models, params);
[rs1 preserved_idx] = prune_nms(rs1, params);

if doflip == 1
  params.detect_add_flip = 1;
  [rs2, t2] = esvm_detectdriver(I, models, params);
  [rs2 preserved_idx] = prune_nms(rs2, params);
else %If there is no flip, then we are done
  resstruct = rs1;
  feat_pyramid = t1;
  return;
end

%If we got here, then the flip was turned on and we need to concatenate
%results
for q = 1:length(rs1.bbs)
  rs1.xs{q} = cat(2,rs1.xs{q}, rs2.xs{q});
  rs1.bbs{q} = cat(1,rs1.bbs{q},rs2.bbs{q});
end
resstruct = rs1;
resstruct.timeSpent = mean([rs1.timeSpent rs2.timeSpent]);
%Concatenate normal and LR pyramids
feat_pyramid = cat(1,t1,t2);

function [resstruct,t] = esvm_detectdriver(I, models, params)
[resstruct,t] = esvm_detectdriverBLOCK(I, models, params);
xs = cell(length(resstruct.xs),1);
for i = 1:length(resstruct.xs)
    for j = 1:size(resstruct.xs{i},2)
        xs{i}{1,j} = resstruct.xs{i}(:,j);
    end
end
resstruct.xs = xs;
return;

function [resstruct,t] = esvm_detectdriverBLOCK(I, models, ...
                                             params)

%%HERE is the chunk version of exemplar localization
U = params.rankerRecipe.U;
V = params.rankerRecipe.V;
bs = params.rankerRecipe.regressorB;
N = size(V,1);
tempSize = params.tempSize;

S = [max(tempSize(:,1)) max(tempSize(:,2))];
fsize = params.init_params.features();
sbin = models{1}.model.init_params.sbin;

if isfield(params,'precomputation')
    if (params.detect_add_flip == 1)
        t = params.precomputation.flip;
    else
        t = params.precomputation.orig;
    end
else
    t = get_pyramid(I, sbin, params);
end


resstruct.padder = t.padder;
available_size = cellfun(@(x)[size(x,1) size(x,2)]-S+1,t.hog,'uniformoutput',false);
available_size = cat(1,available_size{:});
val_idx = find(sum(available_size<=0,2)==0);

pyr_N = prod(available_size(val_idx,:),2);
sumN = sum(pyr_N);
X = zeros(S(1)*S(2)*fsize,sumN);
offsets = cell(length(val_idx), 1);
uus = cell(length(val_idx),1);
vvs = cell(length(val_idx),1);
counter = 1;
for i = 1:length(val_idx)
  s = size(t.hog{val_idx(i)});
  NW = s(1)*s(2);
  ppp = reshape(1:NW,s(1),s(2));
  curf = reshape(t.hog{val_idx(i)},[],fsize);
  b = im2col(ppp,[S(1) S(2)]);
  offsets{i} = b(1,:);
  offsets{i}(end+1,:) = val_idx(i);
  for j = 1:size(b,2)
   X(:,counter) = reshape(curf(b(:,j),:),[],1);
   counter = counter + 1;
  end
  [uus{i},vvs{i}] = ind2sub(s,offsets{i}(1,:));
end
offsets = cat(2,offsets{:});
uus = cat(2,uus{:});
vvs = cat(2,vvs{:});

startTime = tic;
r = V*(U'*X);
r = bsxfun(@plus, r, bs);
[sortedEstRVal sortedEstRIdx] = psort(-vec(r),round(params.reEstRatio*numel(r)));
[recTopModelIdx recTopWindowIdx] = ind2sub(size(r),double(sortedEstRIdx));
reEstR = evalTemplateFromParts({params.W},recTopModelIdx,X,...
    recTopWindowIdx',4);
reEstR = bsxfun(@minus, reEstR, params.bs(recTopModelIdx));
r = -inf(size(r));
r(sortedEstRIdx) = reEstR;
resstruct.timeSpent = toc(startTime);
resstruct.r = max(r,[],2);
resstruct.bbs = cell(N,1);
resstruct.xs = cell(N,1);

if isempty(X)
    return;
end
for exid = 1:N
  goods = find(r(exid,:) >= params.detect_keep_threshold);
  if isempty(goods)
    continue    
  end
  
  [sorted_scores,bb] = ...
      psort(-r(exid,goods)',...
            min(params.detect_max_windows_per_exemplar, ...
                length(goods)));
  bb = goods(bb);
  sorted_scores = -sorted_scores';
  resstruct.xs{exid} = X(:,bb);
  levels = offsets(2,bb);
  scales = t.scales(levels);
  curuus = uus(bb);
  curvvs = vvs(bb);
  
  o = [curuus' curvvs'] - t.padder;
  bbs = ([o(:,2) o(:,1) o(:,2)+tempSize(exid,2) ...
           o(:,1)+tempSize(exid,1)] - 1) .* ...
             repmat(sbin./scales',1,4) + 1 + repmat([0 0 -1 ...
                    -1],length(scales),1);
  
  bbs(:,5:12) = 0;
  bbs(:,5) = (1:size(bbs,1));
  bbs(:,6) = exid;
  bbs(:,8) = scales;
  bbs(:,9) = uus(bb);
  bbs(:,10) = vvs(bb);
  bbs(:,12) = sorted_scores;
  
  if (params.detect_add_flip == 1)
    bbs = flip_box(bbs,t.size);
    bbs(:,7) = 1;
  end
  
  resstruct.bbs{exid} = bbs;
end

if params.detect_save_features == 0
  resstruct.xs = cell(N,1);
end

function [rs preserved_idx] = prune_nms(rs, params)
%Prune via nms to eliminate redundant detections

%If the field is missing, or it is set to 1, then we don't need to
%process anything.  If it is zero, we also don't do NMS.
preserved_idx = [];
if ~isfield(params,'detect_exemplar_nms_os_threshold') || (params.detect_exemplar_nms_os_threshold >= 1) ...
      || (params.detect_exemplar_nms_os_threshold == 0)
  return;
end

[rs.bbs preserved_idx] = cellfun(@(x)esvm_nms(x,params.detect_exemplar_nms_os_threshold),...
    rs.bbs,'uniformoutput',false);

if ~isempty(rs.xs)
  for i = 1:length(rs.bbs)
    if ~isempty(rs.xs{i})
      %NOTE: the fifth field must contain elements
      rs.xs{i} = rs.xs{i}(:,rs.bbs{i}(:,5) );
    end
  end
end

function t = get_pyramid(I, sbin, params)
%Extract feature pyramid from variable I (which could be either an image,
%or already a feature pyramid)

if isnumeric(I)
  if (params.detect_add_flip == 1)
    I = flip_image(I);
  else    
    %take unadulterated "aka" un-flipped image
  end
  
  clear t
  t.size = size(I);

  %Compute pyramid
  [t.hog, t.scales] = esvm_pyramid(I, params);
  t.padder = params.detect_pyramid_padding;
  for level = 1:length(t.hog)
    t.hog{level} = padarray(t.hog{level}, [t.padder t.padder 0], 0);
  end
  
  minsizes = cellfun(@(x)min([size(x,1) size(x,2)]), t.hog);
  t.hog = t.hog(minsizes >= t.padder*2);
  t.scales = t.scales(minsizes >= t.padder*2);  
else
  fprintf(1,'Already found features\n');
  
  if iscell(I)
    if params.detect_add_flip==1
      t = I{2};
    else
      t = I{1};
    end
  else
    t = I;
  end
end

