#include "mex.h"
#include "blas.h"
#include <pthread.h>
#include <math.h>
#include <string.h>
#include <iostream>
#include <vector>


struct thread_data {
    const mxArray* wsCell;
    double* doubleResp;
    const double* modelIdx;
    double* feat;
    const double* quanWindowIdx;
    const mwSize* quanWindowIdxDim;
    const mwSize* partModelDim;
};


void partConvolute(double* doubleResp,double* cur_ws,const double* modelIdx,const mwSize* partModelDim,double* feat,const double* quanWindowIdx,const mwSize* quanWindowIdxDim,int dispIdx){
    // add response for one displacement with cache strategy
    //mexPrintf("NAct=%lu\n",NAct);
    long featFinalIdx;
    long wFinalIdx;
    double* featFinalPtr;
    double* wFinalPtr;
    double res;

    ptrdiff_t dpLength = partModelDim[0];
    ptrdiff_t incx = 1;
    ptrdiff_t incy = 1;


    for (int i=0;i<quanWindowIdxDim[1];i++){
        featFinalIdx = quanWindowIdx[i*quanWindowIdxDim[0]+dispIdx];
        featFinalPtr = (featFinalIdx-1)*partModelDim[0]+feat;
        wFinalIdx = modelIdx[i];
        wFinalPtr = (wFinalIdx-1)*partModelDim[0]+cur_ws;
        res = ddot(&dpLength,featFinalPtr,&incx,wFinalPtr,&incy);
        doubleResp[i] = doubleResp[i] + res;
    }
}

void *threadProcess(void *thread_arg) {
    thread_data *args = (thread_data*)thread_arg;
    const mxArray* wsCell = args->wsCell;
    double* doubleResp = args->doubleResp;
    const double* modelIdx = args->modelIdx;
    double* feat = args->feat;
    const double* quanWindowIdx = args->quanWindowIdx;
    const mwSize* quanWindowIdxDim = args->quanWindowIdxDim;
    const mwSize* partModelDim = args->partModelDim;
    const mwSize NParts = mxGetNumberOfElements(wsCell);

    double* cur_ws;
    for(int i = 0;i<NParts;i++){
        //mexPrintf("I am about to start the %d th iter\n",i);
        mxArray* cur_ws_ptr = mxGetCell(wsCell,i);
        cur_ws = (double*)mxGetPr(cur_ws_ptr);
        //processLut(doubleResp,cur_lut,lut_dims,wordIdx,wordIdx_dims,indexingArr,ActArr,NAct,i);
        partConvolute(doubleResp,cur_ws,modelIdx,partModelDim,feat,quanWindowIdx,quanWindowIdxDim,i);
        //mexPrintf("I finished the %d th iter\n",i);
    }

}

// fast version of template evaluation (for re-estimation)
// reEstR = evalTemplateFromParts(ws,modelIdx,X,quanWindowIdx,numOfThread);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    if (nrhs != 5) {
    mexErrMsgTxt("evalTemplateFromParts.cc: Wrong number of inputs");
    }
    if (nlhs != 1)
    mexErrMsgTxt("evalTemplateFromParts.cc: Wrong number of outputs");
    // get AOMP
    const mxArray* wsCell = prhs[0];
    if (mxGetClassID(wsCell) != mxCELL_CLASS)
        mexErrMsgTxt("evalTemplateFromParts.cc: Invalid input ws, must be a cell array\n");
    const mwSize NParts = mxGetNumberOfElements(wsCell);
    const double* modelIdx = (double*)mxGetPr(prhs[1]);
    const mwSize* modelIdxDim = mxGetDimensions(prhs[1]);
    const mwSize NReEstModels = modelIdxDim[0];
    double* feat = (double*)mxGetPr(prhs[2]);
    const mwSize* featDim = mxGetDimensions(prhs[2]);
    const double* quanWindowIdx = (double*)mxGetPr(prhs[3]);
    const mwSize* quanWindowIdxDim = mxGetDimensions(prhs[3]);
    const int numOfThread = (int)mxGetScalar(prhs[4]);

    mxArray* ws0 = mxGetCell(wsCell, 0);
    if (ws0 == NULL)
        mexErrMsgTxt("evalTemplateFromParts.cc: ws has no content\n");
    const mwSize* partModelDim = mxGetDimensions(ws0);
    if (NParts != quanWindowIdxDim[0])
        mexErrMsgTxt("evalTemplateFromParts.cc: Dimension mismatch between ws and quanWindowIdx\n");

    mxArray* mxResp = mxCreateNumericArray(1, &NReEstModels, mxDOUBLE_CLASS, mxREAL);
    double* doubleResp = (double*)mxGetPr(mxResp);

    // multi-thread
    thread_data *td = (thread_data *)mxCalloc(numOfThread, sizeof(thread_data));
    pthread_t *ts = (pthread_t *)mxCalloc(numOfThread, sizeof(pthread_t));
    long unitN = (long)ceil(double(NReEstModels)/double(numOfThread));
    //mexPrintf("The unitN=%lu\n",unitN);
    long threadCounter = 0;
    long threadEndingPoint;
    int actualThreadNum = 0;
    int stopFlag = 0;
    mwSize** cur_quanWindowIdxDim_chain = (mwSize**)mxCalloc(numOfThread, sizeof(mwSize*));
    for (int i=0;i<numOfThread;i++){
        if (threadCounter>=NReEstModels | stopFlag == 1)
            break;
        if (threadCounter+unitN>=NReEstModels)
            threadEndingPoint = NReEstModels;
        else
            threadEndingPoint = threadCounter + unitN;


        td[i].wsCell = wsCell;
        td[i].feat = feat;
        td[i].partModelDim = partModelDim;

        td[i].doubleResp = doubleResp+threadCounter;
        td[i].modelIdx = modelIdx+threadCounter;
        td[i].quanWindowIdx = quanWindowIdx+quanWindowIdxDim[0]*threadCounter;
        mwSize* cur_quanWindowIdxDim = (mwSize*)mxCalloc(2, sizeof(mwSize));
        cur_quanWindowIdxDim[0] = quanWindowIdxDim[0];
        cur_quanWindowIdxDim[1] = (mwSize)(threadEndingPoint-threadCounter);
        td[i].quanWindowIdxDim = cur_quanWindowIdxDim;
        cur_quanWindowIdxDim_chain[i] = cur_quanWindowIdxDim;
        //mexPrintf("threadCounter=%lu,processing range=%d\n",threadCounter,threadEndingPoint-threadCounter);
        //mexPrintf("cur_wordIdx_dims 1:%d,cur_wordIdx_dims 2:%d\n",cur_wordIdx_dims[0],cur_wordIdx_dims[1]);
        //mexPrintf("address for cur_wordIdx_dims: %p\n",td[i].wordIdx_dims);
        threadCounter = threadEndingPoint;
        if (pthread_create(&ts[i], NULL, threadProcess, (void *)&td[i])){
            stopFlag = 1;
            mexErrMsgTxt("evalTemplateFromParts.cc: Error creating thread");
        }
        else{
            actualThreadNum = actualThreadNum + 1;
        }
    }

    void *status;
    for (int i=0;i<actualThreadNum;i++){
        pthread_join(ts[i], &status);
        mxFree(cur_quanWindowIdxDim_chain[i]);
    }
    plhs[0] = mxResp;
    mxFree(cur_quanWindowIdxDim_chain);
}


