#include "mex.h"
#include "blas.h"
#include <pthread.h>
#include <math.h>
#include <string.h>
#include <iostream>
#include <vector>

struct thread_data {
    const mxArray* lutCellPtr;
    double* doubleResp;
    double* wordIdx;
    const mwSize* lut_dims;
    const mwSize* wordIdx_dims;
    long* indexingArr;
    double* ActArr;
    const long* NAct;
};


void createIndexing(const double* AOMP,const mwSize* AOMP_dims,const long NAct,long* indexingArr,double* ActArr){
    long counter = 0;
    long outerCounter = 0;
    long interCounter;
    for (int i =0;i<AOMP_dims[1];i++)
    {
        interCounter = 0;
        for (int j = 0;j<AOMP_dims[0];j++)
        {
            if (AOMP[j+counter]!=0)
            {
                indexingArr[outerCounter] = j;
                ActArr[outerCounter] = AOMP[j+counter];
                outerCounter = outerCounter + 1;
                interCounter = interCounter + 1;
            }
        }
        if (interCounter<NAct)
        {
            for (int j =0;j<NAct-interCounter;j++)
            {
                indexingArr[outerCounter] = -1;
                ActArr[outerCounter] = -1;
                outerCounter = outerCounter + 1;
            }
        }
        else if (interCounter>NAct)
        {
            mexErrMsgTxt("fastSparseMultiplication.cc: The # of activation in the codes is larger than specified");
        }
        counter = counter + AOMP_dims[0];
    }
    //DEBUG
    //for (int i=0;i<outerCounter;i++){
    //    mexPrintf("%lu\n",indexingArr[i]);
    //}
    //mexPrintf("%lu length for indexingArr\n",outerCounter);
}

//void setDoubleArray(double* TOArray, double* FROMArray, mwSize t){
//    for(long i = 0;i<t;i++){
//        //mexPrintf("copy the %lu th scalar\n",i);
//        TOArray[i]=FROMArray[i];
//    }
//}

void processLut(double* resp,double* lut,const mwSize* lut_dims,double* wordIdx,const mwSize* wordIdx_dims, long* indexingArr,double* ActArr,long NAct,int dispIdx){
    // add response for one displacement with cache strategy
    //mexPrintf("NAct=%lu\n",NAct);
    //const long wordIdxStartingIdx = wordIdx_dims[0]*dispIdx;
    long word;
    long resp_counter = 0;
    long AOMPStartingIdx;
    long lutColIdx;
    double lutColMultiplier;

    //ptrdiff_t n = lut_dims[0];
    //ptrdiff_t incy = 1;

    for (int i = 0;i<wordIdx_dims[1];i++){
        word = wordIdx[i*wordIdx_dims[0]+dispIdx];
        AOMPStartingIdx = (word-1)*NAct;
        //mexPrintf("word=%lu,AOMPStartingIdx=%lu\n",word,AOMPStartingIdx);
        for (int j = 0;j<NAct;j++){
            lutColIdx = indexingArr[AOMPStartingIdx+j];
            if (lutColIdx<0) // No activations here
                continue;
            lutColMultiplier = ActArr[AOMPStartingIdx+j];
            for (int k = 0;k<lut_dims[0];k++){
                resp[resp_counter+k] = resp[resp_counter+k] + lutColMultiplier*lut[lutColIdx*lut_dims[0]+k];
            }
            //daxpy(&n,&lutColMultiplier,lut+lutColIdx*lut_dims[0],&incy,resp+resp_counter,&incy);
        }
        resp_counter = resp_counter + lut_dims[0];
    }
}

void *threadProcess(void *thread_arg) {
    thread_data *args = (thread_data*)thread_arg;
    const mxArray* lutCellPtr = args->lutCellPtr;
    double* doubleResp = args->doubleResp;
    double* wordIdx = args->wordIdx;
    const mwSize* lut_dims = args->lut_dims;
    const mwSize* wordIdx_dims = args->wordIdx_dims;
    long* indexingArr = args->indexingArr;
    double* ActArr = args->ActArr;
    const long NAct = args->NAct[0];

    const mwSize num_lut = mxGetNumberOfElements(lutCellPtr);
    double* cur_lut;
    //mexPrintf("num_lut=%d\n",num_lut);
    for(int i = 0;i<num_lut;i++){
        //mexPrintf("I am about to start the %d th iter\n",i);
        mxArray* cur_lut_ptr = mxGetCell(lutCellPtr,i);
        //cur_lut = prepare_lut_double((double*)mxGetPr(cur_lut_ptr),lut_dims);
        cur_lut = (double*)mxGetPr(cur_lut_ptr);
        processLut(doubleResp,cur_lut,lut_dims,wordIdx,wordIdx_dims,indexingArr,ActArr,NAct,i);
        //mexPrintf("I finished the %d th iter\n",i);
    }

}

// fast version of sparse multiplication
// r = fastSparseMultiplication(AOMP,wordIdx,lutStruct.lut,NAct,numOfThread);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    if (nrhs != 5) {
    mexErrMsgTxt("fastSparseMultiplication.cc: Wrong number of inputs");
    }
    if (nlhs != 1)
    mexErrMsgTxt("fastSparseMultiplication.cc: Wrong number of outputs");
    // get AOMP
    const mxArray* mxAOMP = prhs[0];
    const mwSize* AOMP_dims = mxGetDimensions(mxAOMP);
    const double* AOMP = (double*)mxGetPr(mxAOMP);
    const long NAct = mxGetScalar(prhs[3]);
    const int numOfThread = mxGetScalar(prhs[4]);

    // get wordIdx
    const mxArray* wordIdx = prhs[1];
    const mwSize* wordIdx_dims = mxGetDimensions(wordIdx);
    // get lut
    const mxArray* lutCellPtr = prhs[2];


    if (mxGetClassID(lutCellPtr) != mxCELL_CLASS)
        mexErrMsgTxt("fastSparseMultiplication.cc: Invalid input lut, must be a cell array\n");
    const mwSize num_lut = mxGetNumberOfElements(lutCellPtr);
    mxArray* lut0 = mxGetCell(lutCellPtr, 0);
    if (lut0 == NULL)
        mexErrMsgTxt("fastSparseMultiplication.cc: lut has no content\n");
    const mwSize* lut_dims = mxGetDimensions(lut0);
    if (num_lut != wordIdx_dims[0])
        mexErrMsgTxt("fastSparseMultiplication.cc: Dimension mismatch between lut and wordIdx\n");

    // create the sparse indexing
    long* indexingArr = (long*)mxCalloc(NAct*AOMP_dims[1],sizeof(long));
    double* ActArr = (double*)mxCalloc(NAct*AOMP_dims[1],sizeof(double));
    createIndexing(AOMP,AOMP_dims,NAct,indexingArr,ActArr);
    //mexPrintf("I finished indexing process\n");

    double* curWordIdx = (double*)mxGetPr(wordIdx);
    mwSize* resp_dims = (mwSize*)mxCalloc(2,sizeof(mwSize));
    resp_dims[0] = lut_dims[0];
    resp_dims[1] = wordIdx_dims[1];
    mxArray* mxResp = mxCreateNumericArray(2, resp_dims, mxDOUBLE_CLASS, mxREAL);
    double* doubleResp = (double*)mxGetPr(mxResp);

    // multi-thread
    thread_data *td = (thread_data *)mxCalloc(numOfThread, sizeof(thread_data));
    pthread_t *ts = (pthread_t *)mxCalloc(numOfThread, sizeof(pthread_t));
    long unitN = (long)ceil(double(wordIdx_dims[1])/double(numOfThread));
    //mexPrintf("The unitN=%lu\n",unitN);
    long threadCounter = 0;
    long threadEndingPoint;
    int actualThreadNum = 0;
    int stopFlag = 0;
    mwSize** cur_wordIdx_dims_chain = (mwSize**)mxCalloc(numOfThread, sizeof(mwSize*));
    for (int i=0;i<numOfThread;i++){
        if (threadCounter>=wordIdx_dims[1] | stopFlag == 1)
            break;
        if (threadCounter+unitN>=wordIdx_dims[1])
            threadEndingPoint = wordIdx_dims[1];
        else
            threadEndingPoint = threadCounter + unitN;
        td[i].lutCellPtr = lutCellPtr;
        td[i].lut_dims = lut_dims;
        td[i].indexingArr = indexingArr;
        td[i].ActArr = ActArr;
        td[i].NAct = &NAct;

        td[i].doubleResp = doubleResp+lut_dims[0]*threadCounter;
        td[i].wordIdx = curWordIdx+wordIdx_dims[0]*threadCounter;
        mwSize* cur_wordIdx_dims = (mwSize*)mxCalloc(2, sizeof(mwSize));
        cur_wordIdx_dims[0] = wordIdx_dims[0];
        cur_wordIdx_dims[1] = (mwSize)(threadEndingPoint-threadCounter);
        //mexPrintf("threadCounter=%lu,processing range=%d\n",threadCounter,threadEndingPoint-threadCounter);
        //mexPrintf("cur_wordIdx_dims 1:%d,cur_wordIdx_dims 2:%d\n",cur_wordIdx_dims[0],cur_wordIdx_dims[1]);
        td[i].wordIdx_dims = cur_wordIdx_dims;
        cur_wordIdx_dims_chain[i] = cur_wordIdx_dims;
        //mexPrintf("address for cur_wordIdx_dims: %p\n",td[i].wordIdx_dims);
        threadCounter = threadEndingPoint;
        if (pthread_create(&ts[i], NULL, threadProcess, (void *)&td[i])){
            stopFlag = 1;
            mexErrMsgTxt("fastSparseMultiplication.cc: Error creating thread");
        }
        else{
            actualThreadNum = actualThreadNum + 1;
        }
    }

    void *status;
    for (int i=0;i<actualThreadNum;i++){
        pthread_join(ts[i], &status);
        //free memory for cur_wordIdx_dims
        mxFree(cur_wordIdx_dims_chain[i]);
    }


    //double* cur_lut;
    //for(int i = 0;i<num_lut;i++){
    //    //mexPrintf("I am about to start the %d th iter\n",i);
    //    mxArray* cur_lut_ptr = mxGetCell(lutCellPtr,i);
    //    //cur_lut = prepare_lut_double((double*)mxGetPr(cur_lut_ptr),lut_dims);
    //    cur_lut = (double*)mxGetPr(cur_lut_ptr);
    //    processLut(doubleResp,cur_lut,lut_dims,curWordIdx,wordIdx_dims,indexingArr,ActArr,NAct,i);
    //    //mexPrintf("I finished the %d th iter\n",i);
    //}

    plhs[0] = mxResp;
    // free memory
    mxFree(indexingArr);
    mxFree(ActArr);
    mxFree(resp_dims);
    mxFree(cur_wordIdx_dims_chain);
}

