function interactive_analyze(original_img,bboxes,estimated_responses,dispK,models,imgs_folder)
% function for interactive analyzing results of model recommendation
if sum(bboxes(:,end)<0)>0
    error('The scores should be positive reals');
end

N_models = length(models);
[sortval recommended_idx] = sort(estimated_responses,'descend');
% first let's display results from recommended_idx
recommended_idx = recommended_idx(1:min(length(recommended_idx),dispK));
flip_flag_idx = recommended_idx>N_models;
recommended_idx(flip_flag_idx) = recommended_idx(flip_flag_idx) - N_models;

gap = .5;
set(0,'Units','pixels') 
scnsize = get(0,'ScreenSize');
pos1 = [0.5,0.5,scnsize(3)/2-gap,scnsize(4)];
pos2 = [scnsize(3)/2+gap,0.5,scnsize(3)/2,scnsize(4)];
    
h1 = figure;
set(h1,'OuterPosition',pos1) 
for i = 1:length(recommended_idx)
    recommended_img = imread(fullfile(imgs_folder,...
        [models{recommended_idx(i)}.curid '.png']));
    recommended_img = plot_bb(recommended_img,...
        models{recommended_idx(i)}.gt_box,'r');
    if flip_flag_idx(i)
        recommended_img = flip_image(recommended_img);
    end
    subplot_tight(5,2,i);
    imshow(recommended_img);
    title(sprintf('%d/%d, est val=%.3f',i,...
        length(recommended_idx),sortval(i)));
end

% now display the response map and provide interactive interface
% generate heat map
img_height = size(original_img,1);
img_width = size(original_img,2);
bboxes(:,1:4) = clip_to_image(round(bboxes(:,1:4)),[1 1 img_width img_height]);
heat = zeros(img_height,img_width);
max_map = zeros(img_height,img_width);
for i = 1:size(bboxes,1)
    update_flag = bboxes(i,end)>heat(bboxes(i,2):bboxes(i,4),bboxes(i,1):bboxes(i,3));
    template = heat(bboxes(i,2):bboxes(i,4),bboxes(i,1):bboxes(i,3));
    template(update_flag) = bboxes(i,end);
    heat(bboxes(i,2):bboxes(i,4),bboxes(i,1):bboxes(i,3)) = template;
    model_idx = bboxes(i,6)+bboxes(i,7)*N_models;
    template = max_map(bboxes(i,2):bboxes(i,4),bboxes(i,1):bboxes(i,3));
    template(update_flag)  = model_idx;
    max_map(bboxes(i,2):bboxes(i,4),bboxes(i,1):bboxes(i,3)) = template;
end
[max_point max_coord] = max(heat(:));
[maxVer maxHor] = ind2sub(size(heat),max_coord);

curVer = maxVer;
curHor = maxHor;
h2 = figure;
set(h2,'OuterPosition',pos2) 
while 1
    subplot_tight(3,1,1);
    imshow(original_img);
    hold on;
    plot(curHor,curVer,'b.','MarkerSize',20);
    hold off;
    
    subplot_tight(3,1,2);
    imagesc(heat);
    hold on;
    plot(curHor,curVer,'b.','MarkerSize',20);
    colormap;
    hold off;
    axis off
    if max_map(curVer,curHor)~=0
        model_idx = max_map(curVer,curHor);
        if model_idx>N_models
            model_idx = model_idx - N_models;
            flip_flag = 1;
        else
            flip_flag = 0;
        end
        title(sprintf('x=%d, y=%d, modelIdx=%d, response=%.3f',...
            curVer,curHor,model_idx,heat(curVer,curHor)));
        model_img = imread(fullfile(imgs_folder,[models{model_idx}.curid '.png']));
        model_img = plot_bb(model_img,models{model_idx}.gt_box,'r');
        if flip_flag
            model_img = flip_image(model_img);
        end
    else
        model_img = zeros(img_height,img_width,3);
    end
    subplot_tight(3,1,3);
    imshow(model_img);
    
    [curHor curVer button] = ginput(1);
    if button == 3
        close all;
        return;
    end
    curVer = min(max(round(curVer),1),img_height);
    curHor = min(max(round(curHor),1),img_width);
end





