function [sampledFeats sampledUus sampledVvs sampledLayers sampledScales sampledBboxes] ...
    = sampleFromHogPyramid(I,NSamples,init_params)
% Initialize the model representing patches. Therefore the key difference
% between this and the standard initialization procedure is the size of the
% model is determined by the average size of the common patches.
% Fanyi Xiao

sampledFeats = [];
sampledUus = [];
sampledVvs = [];
sampledLayers = [];
sampledScales = [];

if ~exist('init_params','var')
  init_params.sbin = 8;
  init_params.hg_size = [10 10];
  init_params.MAXDIM = 10;
end
if ~isfield(init_params,'template_length')
    init_params.template_length=10;
end


if ~isfield(init_params,'MAXDIM')
  init_params.MAXDIM = 8;
  fprintf(1,'Default MAXDIM is %d\n',init_params.MAXDIM);
end

epsilon = 1e-8;

template_length = init_params.template_length;
%Get the hog feature pyramid for the entire image
clear params;
params.detect_levels_per_octave = 10;
params.init_params = init_params;
fsize = params.init_params.features();
[f_real,scales] = esvm_pyramid(I, params);

% hack: to remove the outside padding
for i = 1:length(f_real)
    f_real{i}(1,:,:)=[];
    f_real{i}(end,:,:)=[];
    f_real{i}(:,1,:)=[];
    f_real{i}(:,end,:)=[];
end

% compute the probability of sampling from different layers
available_size = cellfun(@(x)[size(x,1) size(x,2)]-template_length+1,f_real,'uniformoutput',false);
available_size = cat(1,available_size{:});
val_idx = find(sum(available_size<=0,2)==0);
if isempty(val_idx)
    return;
end

pyr_N = prod(available_size(val_idx,:),2);
sumN = sum(pyr_N);
X = zeros(template_length^2*fsize,sumN);
offsets = cell(length(val_idx), 1);
uus = cell(length(val_idx),1);
vvs = cell(length(val_idx),1);
counter = 1;
for i = 1:length(val_idx)
  s = size(f_real{val_idx(i)});
  NW = s(1)*s(2);
  ppp = reshape(1:NW,s(1),s(2));
  curf = reshape(f_real{val_idx(i)},[],fsize);
  b = im2col(ppp,[template_length template_length]);
  offsets{i} = b(1,:);
  offsets{i}(end+1,:) = val_idx(i);
  for j = 1:size(b,2)
   X(:,counter) = reshape(curf(b(:,j),:),[],1);
   counter = counter + 1;
  end
  [uus{i},vvs{i}] = ind2sub(s,offsets{i}(1,:));
end
offsets = cat(2,offsets{:});
uus = cat(2,uus{:});
vvs = cat(2,vvs{:});
sampledIdx = randperm(size(X,2));
sampledIdx = sampledIdx(1:min(NSamples,length(sampledIdx)));

sampledFeats = X(:,sampledIdx);
sampledUus = uus(:,sampledIdx);
sampledVvs = vvs(:,sampledIdx);
sampledLayers = offsets(2,sampledIdx);
sampledScales = scales(sampledLayers);
o = [sampledUus' sampledVvs'];
sampledBboxes = ([o(:,2) o(:,1) o(:,2)+template_length ...
       o(:,1)+template_length]) .* ...
         repmat(init_params.sbin./sampledScales',1,4) + 1 + repmat([0 0 -1 ...
                -1],length(sampledScales),1);

debug = 0;
if debug
    dispI = plot_bb(I,bbox,'r');
    dispHOG = HOGpicture(curfeats);
    subplot_tight(1,2,1);
    imshow(dispI);
    title('Image');
    subplot_tight(1,2,2);
    imshow(dispHOG);
    title('HOG');
end







