% Softmax Regression
% Author: Fanyi Xiao, 7/11/2012

function model = softmax(lambda, data, labels,isStoch,model)
% data: contains samples in each column
% k: number of classes
% lambda: the parameter decay coefficient
% labels: M*1 vector consists of labels of the training set

% params
DEBUG = 0;

if ~exist('model','var') || isempty(model)
    ind = unique(labels);
else
    ind = model.ind;
end

new_labels = -ones(length(labels),1);
for i = 1:length(ind)
    valIdx = ind(i) == labels;
    new_labels(valIdx) = i;
end
% check if all labels have been assigned new labels
if sum(new_labels<0)>0
    error('Unknow labels!');
end

k = size(ind,1);
[N M] = size(data);
% Initialization
if ~exist('model','var') || isempty(model)
    theta = rand(N*k,1);
else
    theta = model.theta(:);                                     
end

% augment the data with a 1 vector
% add bias? NO
% N = N + 1;
% data = [ones(1,M);data];


label_matrix = zeros(k,M);
for i = 1:M
    label_matrix(new_labels(i),i) = 1;
end

if DEBUG
    [val grad] = funeval(theta,lambda,data,label_matrix);
    [numgrad] = computeNumericalGradient(@(x)funeval(x,...
        lambda,data,label_matrix), theta);
    disp([grad numgrad]); 
    norm(grad-numgrad)
    max(abs(grad-numgrad))
end

% compute the minimum of the objective using the minFunc
%addpath minFunc/
options.Method = 'lbfgs'; 
options.LS = 1; 
if isStoch % whether stochastic update
    options.maxIter = 20;
else
    options.maxIter = 600;
end
options.display = 'on';
[opttheta, cost] = minFunc( @(p) funeval(p,lambda,data,label_matrix),theta, options);
model.theta = reshape(opttheta,[N k]);
model.ind = ind;

function [val grad] = funeval(theta,lambda,data,label_matrix)
[N M] = size(data);
k = size(label_matrix,1);
theta = reshape(theta,[N k]);

% Using normalization to avoid being numerically unstable
thetaTdata = theta'*data;
[mnum mind] = max(thetaTdata);
thetaTdata_red = thetaTdata - repmat(mnum, [size(thetaTdata,1) 1]);
expval = exp(thetaTdata_red);

% exp_sum = repmat(log(sum(exp(theta'*data),1)),[k 1]);
% val = -1/M*trace(label_matrix'*(theta'*data-exp_sum)) + lambda/2*trace(theta'*theta);

val = -1/M*trace(label_matrix'*(theta'*data-repmat(mnum,[k 1])-repmat(log(sum(expval,1)),[k 1]))) + lambda/2*trace(theta'*theta);
grad = -1/M*data*(label_matrix-expval./repmat(sum(expval,1),[k 1]))' + lambda*theta;
grad = reshape(grad,[N*k 1]);
