function [val grad] = sv_obj(x,data,margin,C)
% 1/2*|x|^2 + \sum_i C/N*max(margin_i - x'*data_i,0)
epsilon = 1e-6;

N = size(data,2);
vio = margin - data'*x;
val = 0.5*norm(x)^2 + C/N*sum(max(vio,0));

nondifferential = abs(vio) < epsilon;
vi_idx = vio>=0;
vi_idx(nondifferential) = false;

grad = x - C/N*sum(data(:,vi_idx),2);

if sum(nondifferential) > 0
    grad_right = - C/N*sum(min(0,data(:,nondifferential)),2);
    grad_left = - C/N*sum(max(0,data(:,nondifferential)),2);

    grad_right = grad_right + grad;
    grad_left = grad_left + grad;
    grad = 0*grad;
        
    left_sg_idx = (grad_left>0);
    right_sg_idx = (grad_right<0);
    grad(left_sg_idx) = grad_left(left_sg_idx);
    grad(right_sg_idx) = grad_right(right_sg_idx);
end