function [h hpt] = visualize_comparison(im,models,M,dets_1,dets_2,COV,bin_COV,...
            method_strings,imgs_folder,dets_folder,filenameList,record_1,record_2,hash_record,h,hpt)
imgIdx = hash_record.imgIdx;
probeModelIdx = hash_record.probeModelIdx;
gtbboxes = hash_record.gtbboxes;
bboxes_filename = fullfile(dets_folder,[filenameList{imgIdx} '.mat']);
% some hard-coded params
hash_record.det_thresh = .9;
hash_record.clusterPerK = 3;
hash_record.os_thresh = .5;
hash_record.clusterK = 20;

status_1 = (record_1>0);
status_2 = (record_2>0);
conflicts = find(status_1~=status_2);
not_diff = find(record_1>=0);
count = 1;

% set up a button
if ~exist('h','var')
    h = figure;
    ht = uitoolbar(h);
    % Use a MATLAB icon for the tool
    [X map] = imread(fullfile(...
        matlabroot,'toolbox','matlab','icons','matlabicon.gif'));
    % Convert indexed image and colormap to truecolor
    icon = ind2rgb(X,map);
    % Create a uipushtool in the toolbar
    hpt = uipushtool(ht,'CData',icon,...
                     'TooltipString','push to view recommendation',...
                     'ClickedCallback',...
                     {@myOnClick,im,bboxes_filename,COV,models,M,imgs_folder,hash_record});
else
    set(hpt,'ClickedCallback',...
                     {@myOnClick,im,bboxes_filename,COV,models,M,imgs_folder,hash_record})
end

% plot all not-difficult bboxes onto im
for i = 1:length(not_diff)
    im = plot_bb(im,gtbboxes(not_diff(i),:),'g');
end

while 1
        if ~isempty(conflicts)
            cur_ptr = conflicts(count);
            if status_1(conflicts(count))>0
                is1 = true;
                method_string = method_strings{1};
            else
                is1 = false;
                method_string = method_strings{2};
            end
            
            if is1
                modelIdx = dets_1(record_1(cur_ptr),6);
                flip_flag = dets_1(record_1(cur_ptr),7);
                det_bbox = dets_1(record_1(cur_ptr),1:4);
                det_resp = dets_1(record_1(cur_ptr),end);
            else
                modelIdx = dets_2(record_2(cur_ptr),6);
                flip_flag = dets_2(record_2(cur_ptr),7);
                det_bbox = dets_2(record_2(cur_ptr),1:4);
                det_resp = dets_2(record_2(cur_ptr),end);
            end
            

            gtbbox = gtbboxes(cur_ptr,:);
            cur_im = plot_bb(im,gtbbox,'r');
            cur_im = plot_bb(cur_im,det_bbox,'b');
            
            modelIm_filename = fullfile(imgs_folder,[models{modelIdx}.curid '.png']);
            modelIm = imread(modelIm_filename);
            model_gtbbox = models{modelIdx}.gt_box;
            modelIm = plot_bb(modelIm,model_gtbbox,'r');
            if flip_flag
                modelIm = flip_image(modelIm);
            end
            
            % visualize 
            subplot_tight(2,1,1);
            imshow(cur_im);
            title(sprintf('%d th image, %d/%d conflict dets, %s succussefully detect the target, resp=%.3f',...
                imgIdx,count,length(conflicts),method_string,det_resp));
            subplot_tight(2,1,2);
            imshow(modelIm);
            title('model');
        else
            %subplot_tight(2,1,1);
            %imshow(blank_img);
            %title(sprintf('%d th image, no conflicts',i));
            %subplot_tight(2,1,2);
            %imshow(blank_img);
            break;
        end
        [count isBreak] = myPause(count,length(conflicts));
        if isBreak; break; end
end


function myOnClick(hObject, eventdata,im,bboxes_filename,COV,models,M,imgs_folder,hash_record)
imgIdx = hash_record.imgIdx;
clusterPerK = hash_record.clusterPerK;
info.gtbboxes = hash_record.gtbboxes;
load(bboxes_filename);
bboxes = test_grid.bboxes;
bboxes(:,11) = imgIdx;
bboxes = esvm_calibrate_boxes(bboxes,M.betas);
info.bboxIdxAfterNMS = hash_record.bboxIdxAfterNMS;
for i = 1:size(hash_record.probeCluster_bboxes,1)
    cur_probe_bboxes = bboxes(hash_record.probeBboxes_idx{i},:);
    recommendedCluster_idx = hash_record.recommendedCluster_idx{i};
    %info.recommendedCluster_idx = recommendedCluster_idx;
    info.recall = hash_record.recall{i};
    info.precision = hash_record.precision{i};
    info.probeCluster_bboxes = hash_record.probeCluster_bboxes(i,:);
    interupt_signal = visualize_clusters(im,cur_probe_bboxes,COV(:,recommendedCluster_idx),info,...
        3,clusterPerK,models,imgs_folder,bboxes);
    if interupt_signal
        return;
    end
end


% % compute the index of detections that are finally selected
% bboxIdxAfterNMS = find(bboxes(:,end)>=det_thresh&ismember(bboxes(:,6),recommended_idx));
% recommended_bboxes = bboxes(bboxIdxAfterNMS,:);
% [recommended_bboxes pick] = esvm_nms(recommended_bboxes,os_thresh);
% bboxIdxAfterNMS = bboxIdxAfterNMS(pick);
% info.dets_selected_idx = bboxIdxAfterNMS;
% % prune out the bboxes with low responses
% pruned_bboxes = bboxes(bboxes(:,end)>=det_thresh,:);
% N_models = length(models);
% selectionSeq = [probeModelIdx probeModelIdx+N_models];
% probe_bboxes = pruned_bboxes(ismember(pruned_bboxes(:,6),probeModelIdx),:);
% % cluster the probe bboxes
% pb_clusters = bboxes_clustering(probe_bboxes,os_thresh);
% u_pb_clusterIdx = unique(pb_clusters);
% 
% 
% for j = 1:length(u_pb_clusterIdx)
%     val_idx = pb_clusters == u_pb_clusterIdx(j);
%     cur_probe_bboxes = probe_bboxes(val_idx,:);
%     
%     modelIdx = cur_probe_bboxes(:,6);
%     modelIdx(cur_probe_bboxes(:,7)==1) = modelIdx(cur_probe_bboxes(:,7)==1) + N_models;
%     u_modelIdx = unique(modelIdx);
%     cur_probe_r = zeros(2*N_models,1);
%     for k = 1:length(u_modelIdx)
%         val_idx = find(u_modelIdx(k) == modelIdx);
%         [maxval maxidx] = max(cur_probe_bboxes(val_idx,end));
%         cur_probe_r(u_modelIdx(k)) = cur_probe_bboxes(val_idx(maxidx),end);
%     end
%     cur_probe_r = cur_probe_r(selectionSeq);
%     hashcode = double(cur_probe_r>0);
%     %coin_count = sum(repmat(hashcode,[1 size(bin_COV,2)]) == ...
%     %   bin_COV(selectionSeq,:),1);
%     coin_count = hashcode'*bin_COV(selectionSeq,:);
%     
%     [sortval sortidx] = sort(coin_count,'descend');
%     recall = sum(bin_COV(selectionSeq(hashcode>0),sortidx(1:clusterK)),1)./sum(hashcode);
%     precision = sum(repmat(hashcode,[1 clusterK]).*bin_COV(selectionSeq,sortidx(1:clusterK)),1)./...
%         sum(bin_COV(selectionSeq,sortidx(1:clusterK)),1);
%     info.cluster_idx = sortidx(1:clusterK);
%     info.recall = recall;
%     info.precision = precision;
%     interupt_signal = visualize_clusters(im,cur_probe_bboxes,COV(:,info.cluster_idx),info,...
%         3,clusterPerK,models,imgs_folder,bboxes);
%     if interupt_signal
%         return;
%     end
% end

