function [model isValid] = esvm_initialize_specsize_model(I,bbox,init_params)
% Initialize the model representing patches. Therefore the key difference
% between this and the standard initialization procedure is the size of the
% model is determined by the average size of the common patches.
% Fanyi Xiao


if ~exist('init_params','var')
  init_params.sbin = 8;
  init_params.hg_size = [8 8];
  init_params.MAXDIM = 10;
end

if ~isfield(init_params,'MAXDIM')
  init_params.MAXDIM = 10;
  fprintf(1,'Default MAXDIM is %d\n',init_params.MAXDIM);
end

% N = length(I);
% if size(bbox,1) ~= N
%     error('The number of bbox is not consistent with the number of images!\n');
% end
epsilon = 1e-8;

template_w = init_params.template_w;
template_h = init_params.template_h;

im_w = size(I,2);
im_h = size(I,1);
rescale_w = 1.5*(init_params.sbin*template_w)/(bbox(3)-bbox(1));
rescale_h = 1.5*(init_params.sbin*template_h)/(bbox(4)-bbox(2));
new_im_w = round(rescale_w*im_w);
new_im_h = round(rescale_h*im_h);
bbox([1 3]) = round(bbox([1 3])*rescale_w);
bbox([2 4]) = round(bbox([2 4])*rescale_h);
bbox = clip_to_image(bbox,[1 1 new_im_w new_im_h]);
cur_I = imresize(I,[new_im_h,new_im_w]);


%Create a blank image with the exemplar inside
Ibox = zeros(size(cur_I,1), size(cur_I,2));    
Ibox(bbox(2):bbox(4), bbox(1):bbox(3)) = 1;

%% NOTE: why was I padding this at some point and now I'm not???
%% ANSWER: doing the pad will create artifical gradients
ARTPAD = 0;
I_real_pad = pad_image(cur_I, ARTPAD);

%Get the hog feature pyramid for the entire image
clear params;
params.detect_levels_per_octave = 10;
params.init_params = init_params;
[f_real,scales] = esvm_pyramid(I_real_pad, params);


%Extract the regions most overlapping with Ibox from each level in the pyramid
[masker,sizer] = get_matching_masks(f_real, Ibox);

%Now choose the mask which is closest to N cells
[targetlvl, mask] = get_ncell_mask([template_h template_w],masker,sizer);
[uu,vv] = find(mask);
curfeats = f_real{targetlvl}(min(uu):max(uu),min(vv):max(vv),:);

Level = size(curfeats,3);
% remove those samples which are not consistent with the desired size
feat_sizes = size(curfeats);
size_err = sum(abs([template_h template_w Level] - feat_sizes),2);
isValid = size_err < epsilon;

% curfeats = cellfun(@(x)x(:),curfeats,'UniformOutput',false);
% curfeats = cat(2,curfeats{:});
% curfeats_m = reshape(mean(curfeats,2),[temp_h temp_w Level]);

if isValid
    model.init_params = init_params;
    model.hg_size = size(curfeats);
    model.mask = logical(ones(model.hg_size(1),model.hg_size(2)));
    fprintf(1,'A model initialized, HOG_size = [%d %d]\n',...
        model.hg_size(1),model.hg_size(2));
    model.w = curfeats - mean(curfeats(:));
    model.b = 0;
    model.rescaledH = new_im_h;
    model.rescaledW = new_im_w;
    model.x = curfeats;
else
    model = [];
    return;
end

%Fire inside self-image to get detection location
[model.bb, model.x] = get_target_bb(model, I_real_pad, init_params);

function [targetlvl,mask] = get_ncell_mask(temp_size, masker,sizer)
%Get a the mask and features, where mask is closest to NCELL cells
%as possible
diff = sum(abs(repmat(temp_size,[size(sizer,1) 1])-sizer),2);
[sortval sortind] = sort(diff,'ascend');

for i = 1:length(sortind)
    flag = 1;
    hog_w = size(masker{sortind(i)},2);
    hog_h = size(masker{sortind(i)},1);
    if hog_w>=temp_size(2) && hog_h>=temp_size(1)
        [uu,vv] = find(masker{sortind(i)});
        min_y = min(uu);
        max_y = max(uu);
        min_x = min(vv);
        max_x = max(vv);
        if (min_y + temp_size(1) - 1) <= hog_h
            max_y = min_y + temp_size(1) - 1;
        elseif (max_y - temp_size(1) + 1) >= 1
            min_y = (max_y - temp_size(1) + 1);
        else
            flag = 0;
            max_y = hog_h;
        end

        if (min_x + temp_size(2) - 1) <= hog_w
            max_x = min_x + temp_size(2) - 1;
        elseif (max_x - temp_size(2) + 1) >= 1
            min_x = (max_x - temp_size(2) + 1);
        else
            flag = 0;
            max_x = hog_w;
        end
        if flag == 1
            mask = masker{sortind(i)}*0;
            targetlvl = sortind(i);
            mask(min_y:max_y,min_x:max_x) = 1;
            return;
        end
    end
end
% must be error when reach here
error('Cannot find any matches with specified size');

function [masker,sizer] = get_matching_masks(f_real, Ibox)
%Given a feature pyramid, and a segmentation mask inside Ibox, find
%the best matching region per level in the feature pyramid

masker = cell(length(f_real),1);
sizer = zeros(length(f_real),2);

for a = 1:length(f_real)
  goods = double(sum(f_real{a}.^2,3)>0);
  
  masker{a} = max(0.0,min(1.0,imresize(Ibox,[size(f_real{a},1) size(f_real{a}, ...
                                                  2)])));
  [tmpval,ind] = max(masker{a}(:));
  masker{a} = (masker{a}>.1) & goods;

  if sum(masker{a}(:))==0
    [aa,bb] = ind2sub(size(masker{a}),ind);
    masker{a}(aa,bb) = 1;
  end
  [uu,vv] = find(masker{a});
  masker{a}(min(uu):max(uu),min(vv):max(vv))=1;
  sizer(a,:) = [range(uu)+1 range(vv)+1];
end

function bbox = expand_bbox(bbox,I)
%Expand region such that is still within image and tries to satisfy
%these constraints best
%requirements: each dimension is at least 50 pixels, and max aspect
%ratio os (.25,4)
for expandloop = 1:10000
  % Get initial dimensions
  w = bbox(3)-bbox(1)+1;
  h = bbox(4)-bbox(2)+1;
  
  if h > w*4 || w < 50
    %% make wider
    bbox(3) = bbox(3) + 1;
    bbox(1) = bbox(1) - 1;
  elseif w > h*4 || h < 50
    %make taller
    bbox(4) = bbox(4) + 1;
    bbox(2) = bbox(2) - 1;
  else
    break;
  end
  
  bbox([1 3]) = cap_range(bbox([1 3]), 1, size(I,2));
  bbox([2 4]) = cap_range(bbox([2 4]), 1, size(I,1));      
end


function [target_bb,target_x] = get_target_bb(model, I, init_params)
%Get the bounding box of the top detection

mmm{1}.model = model;
mmm{1}.model.hg_size = size(model.w);
localizeparams.detect_keep_threshold = -100000.0;
localizeparams.detect_max_windows_per_exemplar = 1;
localizeparams.detect_levels_per_octave = 10;
localizeparams.detect_save_features = 1;
localizeparams.detect_add_flip = 0;
localizeparams.detect_pyramid_padding = 5;
localizeparams.dfun = 0;
localizeparams.max_models_before_block_method = 20;
localizeparams.init_params = init_params;

[rs,t] = esvm_detect(I,mmm,localizeparams);
target_bb = rs.bbs{1}(1,:);
target_x = rs.xs{1}{1};

