function [m,other] = esvm_update_svm_LR(m,LR)
% Perform SVM learning for a single exemplar model, we assume that
% the exemplar has a set of detections loaded in m.model.svxs and m.model.svbbs
% Durning Learning, we can apply some pre-processing such as PCA or
% dominant gradient projection

other = 'svm';
%if no inputs are specified, just return the suffix of current method
if nargin==0
  m = '-svm';
  return;
end

if ~isfield(m.model,'mask') || isempty(m.model.mask)
  m.model.mask = true(numel(m.model.w),1);
end

if length(m.model.mask(:)) ~= numel(m.model.w)
  m.model.mask = repmat(m.model.mask,[1 1 m.model.hg_size(3)]);
  m.model.mask = logical(m.model.mask(:));
end

mining_params = m.mining_params;
xs = m.model.svxs;
bbs = m.model.svbbs;


%NOTE: MAXSIZE should perhaps be inside of the default_params script?
MAXSIZE = 3500;
if size(xs,2) >= MAXSIZE
  HALFSIZE = MAXSIZE/2;
  %NOTE: random is better than top 5000
  r = m.model.w(:)'*xs;
  [tmp,r] = sort(r,'descend');
  r1 = r(1:HALFSIZE);
  
  r = HALFSIZE+randperm(length(r((HALFSIZE+1):end)));
  r = r(1:HALFSIZE);
  r = [r1 r];
  xs = xs(:,r);
  bbs = bbs(r,:);
end

supery = cat(1,ones(size(m.model.x,2),1),-1*ones(size(xs,2),1));
exemplar = m.model.x;
X_neg = xs;

spos = size(exemplar,2);
sneg = size(X_neg,2);

wpos = mining_params.train_positives_constant;
fprintf_oct(1,' -----\nStarting SVM: dim=%d... #pos=%d, #neg=%d ',...
        size(exemplar,1),spos,sneg);
tic;


% precomputation = LR.precomputation;
% hg_size = m.model.hg_size;

% UD_vec = zeros(prod(hg_size),2*length(UD));
% % get the hard negative vectors
% for j = 1:length(UD)
%     uus_orig = UD{j}(1,1);
%     vvs_orig = UD{j}(1,2);
%     level_orig = UD{j}(1,3);
%     UD_vec(:,2*j-1) = vec(precomputation{j}.orig.hog{level_orig}(uus_orig:uus_orig+hg_size(1)-1,vvs_orig:vvs_orig+hg_size(2)-1,:));
%     
%     uus_flip = UD{j}(2,1);
%     vvs_flip = UD{j}(2,2);
%     level_flip = UD{j}(2,3);
%     UD_vec(:,2*j) = vec(precomputation{j}.flip.hog{level_flip}(uus_flip:uus_flip+hg_size(1)-1,vvs_flip:vvs_flip+hg_size(2)-1,:));
% end

lambda = LR.lambda;
ratio = LR.ratio;
maxIter = LR.maxIter;
tol = LR.tol;


% check if it is LR iteration, if yes then turn on LR regularization.
% otherwise turn it off (FYX)
% Quasi Newton for nonsmooth function
options = [];
options.display = 'none';
options.Method = 'lbfgs';
options.maxFunEvals = 2000;
options.TolX = 1e-20;
    
if LR.isLR
    UD_vec = LR.UD_vec;
    model_mean = LR.model_mean;
    task_mean = LR.task_mean;
    overall_mean = LR.overall_mean;
    rr = LR.r + overall_mean + repmat(model_mean,[1 size(LR.r,2)]) + repmat(task_mean,[size(LR.r,1) 1]);
    rr = vec(rr)';
    rr = rr(LR.val_idx);
    UD_vec = UD_vec(:,LR.val_idx);
    
    %lbfgs_start = tic;
    [wex val_lbfgs] = minFuncNonsmooth(@LR_obj,[vec(m.model.w);m.model.b],options,[exemplar;-1],...
        [X_neg; -ones(1,size(X_neg,2))],[UD_vec;-ones(1,size(UD_vec,2))],rr,...
        wpos*mining_params.train_svm_c,mining_params.train_svm_c,lambda);
    %timegap = toc(lbfgs_start);
    %fprintf_oct(['Time elapsed: ' num2str(timegap), ', obj value: ' num2str(val_lbfgs) '\n']);
    fprintf_oct(['Obj value: ' num2str(val_lbfgs) '\n']);
    
%     % solver using SGD
%     tic
%     [wex b eval_rec] = esvm_LR_SGD(exemplar,X_neg,UD_vec,rr,wpos*mining_params.train_svm_c,...
%         mining_params.train_svm_c, lambda,ratio,maxIter,tol,vec(m.model.w),m.model.b);
%     toc
%     
%     err1 = 0;
%     err2 = 0;
%     if max(eval_rec)>1e6 
%         err1 = 1;
%     end
%     if max(eval_rec(2:end) - eval_rec(1:end-1))>1
%         err2 = 1;
%     end
% 
%     if err1 || err2
%         error(['Optimization deviates..., err1: ' err1 ', err2: ' err2]);
%     end
    
else
%     % solver using libsvm
%     svm_model = libsvmtrain(supery, [exemplar X_neg]',sprintf(['-s 0 -t 0 -c' ...
%                     ' %f -w1 %.9f -q'], mining_params.train_svm_c, wpos));
%     
%     %convert support vectors to decision boundary
%     svm_weights = full(sum(svm_model.SVs .* ...
%                      repmat(svm_model.sv_coef,1, ...
%                             size(svm_model.SVs,2)),1));
% 
%     wex = svm_weights';
%     b = svm_model.rho;
% 
%     if supery(1) == -1
%         wex = wex*-1;
%         b = b*-1;    
%     end
    
    %lbfgs_start = tic;
    [wex val_lbfgs] = minFuncNonsmooth(@LR_obj,[vec(m.model.w);m.model.b],options,[exemplar;-1],...
        [X_neg; -ones(1,size(X_neg,2))],rand(length(exemplar)+1,1),rand(1),...
        wpos*mining_params.train_svm_c,mining_params.train_svm_c,0);
    %timegap = toc(lbfgs_start);
    %fprintf_oct(['Time elapsed: ' num2str(timegap), ', obj value: ' num2str(val_lbfgs) '\n']);
    fprintf_oct(['Obj value: ' num2str(val_lbfgs) '\n']);
end
b = wex(end);
wex = wex(1:end-1);


% [wex b eval_rec] = esvm_LR_SGD(exemplar,X_neg,rand(length(exemplar),1),rand(1),wpos*mining_params.train_svm_c,...
%         mining_params.train_svm_c, 0,ratio,maxIter,tol,vec(m.model.w),m.model.b);
% obj = standard_svm_obj(wex,b,exemplar,X_neg,wpos*mining_params.train_svm_c,mining_params.train_svm_c);
% eval_rec = eval_func([wex;b],[exemplar;-1],[X_neg;-ones(1,size(X_neg,2))],[UD_vec;-ones(1,size(UD_vec,2))],rr,...
%     wpos*mining_params.train_svm_c,mining_params.train_svm_c,lambda);

% % solver using QP
% C = [wpos*mining_params.train_svm_c/lambda; mining_params.train_svm_c/lambda*ones(sneg,1)];
% [wb gamma] = sub_RLMS(rr,[UD_vec; -ones(1,size(UD_vec,2))],1/lambda,C,-1,[[exemplar X_neg];-ones(1,sneg+spos)],...
%     [vec(m.model.w);m.model.b]);
% wex = wb(1:end-1);
% b = wb(end);
% eval_rec = eval_func([wex;b],[exemplar;-1],[X_neg;-ones(1,size(X_neg,2))],[UD_vec;-ones(1,size(UD_vec,2))],rr,...
%     wpos*mining_params.train_svm_c,mining_params.train_svm_c,lambda);
% eval_rec = eval_func([wex;b],[exemplar;-1],[X_neg;-ones(1,size(X_neg,2))],rand(size(exemplar,1)+1,1),0,...
%     wpos*mining_params.train_svm_c,mining_params.train_svm_c,0);



maxpos = max(wex'*m.model.x - b);

fprintf_oct(1,' --- Max positive is %.3f\n',maxpos);
fprintf_oct(1,'SVM iteration took %.3f sec, ',toc);

% extract the results from the trained vector
m.model.w = reshape(wex, size(m.model.w));
m.model.b = b;

% output the training error
err_seq = m.model.w(:)'*[m.model.x m.model.svxs] - b;
err_seq = sum(max(ones(length(supery),1)-supery.*err_seq',0))/length(supery)

r = m.model.w(:)'*m.model.svxs - m.model.b;
svs = find(r >= -1.0000);

if length(svs) == 0
  fprintf_oct(1,' ERROR: number of negative support vectors is 0!\');
  error('Something went wrong');
end


%KEEP (nsv_multiplier * #SV) vectors, but at most max_negatives of them
total_length = ceil(mining_params.train_keep_nsv_multiplier*length(svs));
total_length = min(total_length,mining_params.train_max_negatives_in_cache);

[alpha,beta] = sort(r,'descend');
svs = beta(1:min(length(beta),total_length));
m.model.svxs = m.model.svxs(:,svs);
m.model.svbbs = m.model.svbbs(svs,:);
fprintf_oct(1,' kept %d negatives\n',total_length);

    
function val = eval_func(w,exemplar,X_neg,UD,r,c1,c2,c3)
% 1/2*w^2 + c1*max(1-w^T*exemplar,0) + c2*sum_(k is belonging to negatives) max(1+w^T*X_k,0) + c3/2*||r-w^T*UD||^2
err = r - w'*UD;
val = 0.5*(w(1:end-1)'*w(1:end-1)) + c1*max(1-w'*exemplar,0) + c2*sum(max(1+w'*X_neg,0)) + c3/2*(err*err');


function obj = standard_svm_obj(w,b,exemplar,X_neg,c1,c2)
obj = 0.5*(w'*w) + c1*max(1-(w'*exemplar - b),0) + c2*sum(max(1+(w'*X_neg - b),0));















