function [constraints bboxes info]=inferenceParallel(structure,sw,w,detector,lambda,c_appr,c_proj,c_resp,c_context,extRate,spry,highlightDetK,hogStepSize,attrIdx,integralImgDir,IMDB,imgDir,printDir,pollingDir,isChainNMS)
% THIS IS THE PARALLEL VERSION OF inference function
% NOTE: the structure.data must contain nodes in a BFS traverse order,
% otherwise this will fail


global ds;

structData=structure.data;
nodeIdx=cellfun(@(x)x.nodeIdx,structData);
parentIdx=cellfun(@(x)x.parent,structData);
fprintf('Inference launched: Node#=%g, c_appr=%g, c_proj=%g, lambda=%g\n',length(structData),c_appr,c_proj,lambda);

for i = 1:length(structData)
    structData{i}.sw=sw;
    structData{i}.childList=nodeIdx(parentIdx==structData{i}.nodeIdx);
end

if isempty(highlightDetK)
    highlightDetK=-inf;
end

if dsmapredisopen()
    dsrundistributed('fprintf(''I am testing!\n'');',10);% get how many available nodes are there
    N=length(ds.sys.distproc.availslaves);% number of machines, therefore number of jobs per batch
else
    N=1;
end

% making child-parent pair so that everyone knows what to do
childParentTable=[];
dsupWrap('ds.store.operatingIntegralDir',integralImgDir);
dsupWrap('ds.store.operatingW',w);
dsupWrap('ds.store.operatingDetector',detector);
dsupWrap('ds.store.operatingLambda',lambda);
dsupWrap('ds.store.operatingC_appr',c_appr);
dsupWrap('ds.store.operatingC_proj',c_proj);
dsupWrap('ds.store.operatingC_resp',c_resp);
dsupWrap('ds.store.operatingC_context',c_context);
dsupWrap('ds.store.operatingExtRate',extRate);
dsupWrap('ds.store.operatingSpry',spry);
dsupWrap('ds.store.operatingPollingDir',pollingDir);
dsupWrap('ds.store.operatingStructData',structData);
dsupWrap('ds.store.operatingNodeIdx',nodeIdx);
dsupWrap('ds.store.operatingIMDB',IMDB);
dsupWrap('ds.store.operatingHogStepSize',hogStepSize)
dsupWrap('ds.store.isChainNMS',isChainNMS);

unix(sprintf('rm -r %s',pollingDir));
for k = length(structData):-1:1,
    if structData{k}.parent==0
        % we add a dummy father for root to exploit the same loop
        childParentTable=[childParentTable;structData{k}.nodeIdx structData{end}.nodeIdx];
    else
        childParentTable=[childParentTable;structData{k}.nodeIdx structData{k}.parent];
    end
    if size(childParentTable,1)==N || k==1
        % do distribute inference jobs
        dsupWrap('ds.store.operatingChildParentTable',childParentTable);
        dssaveAndWait;
        dsrundistributed('distributedPassmsg(dsidx)',size(ds.store.operatingChildParentTable,1));
        
        [ignore childIdx]=ismember(childParentTable(:,1),nodeIdx);
        structData(childIdx)=ds.res.structData(childIdx);
        dsdelete('ds.res.structData');
        
        % for j = length(childIdx)% only plot for the longest chain in this batch
        %     % backtrack now
        %     [res ptr score imgIdx pedroHog]=backtrack(structData(childIdx(j):end));
        %     % printout
        %     if ~isempty(res)
        %         printImageBbox(imgIdx,pedroHog,res,score,imgDir,IMDB,fullfile(printDir,'backtrack'),size(res{1},1),ds.store.whitenMu,ds.store.whitenRT,struct('lambda',lambda,'c_appr',c_appr,'c_proj',c_proj,'hogStepSize',hogStepSize));
        %     end
        %     fprintf('Sub-Tree with %g as the root printed\n',structData{childIdx(j)}.nodeIdx);
        % end
        
        childParentTable=[];
    end
end
unix(sprintf('rm -r %s',pollingDir));
dsdelete('ds.store.operatingChildParentTable');
structData{1}=rmfield(structData{1},'lw');
structData{1}=rmfield(structData{1},'lh');
structData{1}=rmfield(structData{1},'lk');
structData{1}=rmfield(structData{1},'lad');
if isfield(structData{1},'toParentObj')
    structData{1}=rmfield(structData{1},'toParentObj');
end
if isfield(structData{1},'toParentApprDist')
    structData{1}=rmfield(structData{1},'toParentApprDist');
end

% backtrack now
[res ptr score imgIdx pedroHog]=backtrack(structData(1:end),isChainNMS);

if isChainNMS
    initInferenceK=10;
    [res selIdx]=chainNMS(res,initInferenceK,score);
    ptr=ptr(selIdx);
    score=score(selIdx);
end

% printout
if ~isempty(res)
    printImageBbox(imgIdx,pedroHog,res,score,imgDir,IMDB,fullfile(printDir,'backtrack'),size(res{1},1),ds.store.whitenMu,ds.store.whitenRT,...
        struct('lambda',lambda,'c_appr',c_appr,'c_proj',c_proj,'c_resp',c_resp,'c_context',c_context,'hogStepSize',hogStepSize,'extRate',extRate,'highlightDetK',highlightDetK));
end
fprintf('Sub-Tree with %g as the root printed\n',structData{1}.nodeIdx);

% % train a detector out of the chain
% trainDetector(imgIdx,res,score,pedroHog,struct('hogStepSize',hogStepSize));

% collect info 
if isChainNMS
    bboxes=cell(length(res),1);
    info=cell(length(res),1);
    constraints=cell(length(res),1);
    for i = 1:length(res)
        [constraints{i} bboxes{i} info{i}]=organizeOutput(structData,imgIdx,res(i),score(i),attrIdx);
    end
else
    [constraints bboxes info]=organizeOutput(structData,imgIdx,res,score,attrIdx);
end


function trainDetector(imgIdx,res,score,pedroHog,params)
% [1-4 bbox, 5-8 bbox on feat, 9 layer index, 10 layer scale, 11 ranker resp, 
%           12 detector resp, 13 appearance dist to parent, 14 nodeIdx, 15 parentNodeIdx, 16 isParentStronger]
[mxv mxi]=max(score);
res=res{mxi};% take the res with the highest score

for i = 1:length(imgIdx)
    hogPyr=pedroHog{i};
    hogScales=hogPyr.scales;
    hogMap=hogPyr.hog{hogScales==res(i,10)};
    bbox=res(i,5:8);
    uu=bbox(2);
    vv=bbox(1);
    SW(1)=bbox(4)-bbox(2)+1;
    SW(2)=bbox(3)-bbox(1)+1;
    hogHMark=1:params.hogStepSize:SW(1);
    hogWMark=1:params.hogStepSize:SW(2);
    hIdx=max(1,min(size(hogMap,1),floor((uu+hogHMark-1)/params.hogStepSize)+1));
    wIdx=max(1,min(size(hogMap,2),floor((vv+hogWMark-1)/params.hogStepSize)+1));
    hogFeat=hogMap(hIdx,wIdx,:);
end



function [constraints bboxes info]=organizeOutput(structData,imgIdx,res,score,attrIdx)
% forming output for next iter ranker learning
nodeIdx=cellfun(@(x)x.nodeIdx,structData);
nodeImgIdx=cellfun(@(x)x.imgIdx,structData);
[maxScore maxIdx]=max(score);
res=res{maxIdx};
bboxes=zeros(size(res,1),7);
info=cell(size(res,1),1);
for i = 1:size(res,1)
    bboxes(i,:)=[res(i,1:4) imgIdx(i) res(i,8)-res(i,6)+1 res(i,7)-res(i,5)+1];
    % collect more info 
    info{i}=structData{imgIdx(i)==nodeImgIdx};
    info{i}=rmfield(info{i},'parent');
    info{i}=rmfield(info{i},'nodeIdx');
    info{i}=rmfield(info{i},'rankerFeat');
    info{i}=rmfield(info{i},'pyrFeat');
    info{i}=rmfield(info{i},'childList');
    info{i}=rmfield(info{i},'score');
    if isfield(info{i},'lh')
        info{i}=rmfield(info{i},'lh');
        info{i}=rmfield(info{i},'lw');
        info{i}=rmfield(info{i},'lk');
        info{i}=rmfield(info{i},'lad');
    end
    info{i}=rmfield(info{i},'pedroHog');
    info{i}.res=res(i,:);
end

[sortVal sortIdx]=sort(res(:,11),'descend');
sortedNodeIdx=res(sortIdx,14);
[ignore corrIdx]=ismember(sortedNodeIdx,nodeIdx);
structData=structData(corrIdx);
constraints=[];
for i = 1:length(structData)-1
    constraints=[constraints; structData{i}.imgIdx structData{i+1}.imgIdx attrIdx 1 1 1 1]; % we set the confidence of all relative attributes to be uniform
end


function printImageBbox(imgIdx,pedroHog,res,score,imgDir,IMDB,printDir,idx,whitenMu,whitenRT,params)
% printout: NOTE that currently assumes all res share the same imgIdx
% res: 
% [1-4 bbox, 5-8 bbox on feat, 9 layer index, 10 layer scale, 11 ranker resp, 
%           12 detector resp, 13 appearance dist to parent, 14 nodeIdx, 15 parentNodeIdx, 16 isParentStronger 17 context dist to parent]
global ds;

topK=100;% display topK tracks
[sortedScore sortIdx]=sort(score,'descend');
res=res(sortIdx(1:min(length(sortIdx),topK)));
score=score(sortIdx(1:min(length(sortIdx),topK)));

% write images
dsupWrap('ds.store.operaingIdx',imgIdx);
dsupWrap('ds.store.operatingIMDB',IMDB);
dsupWrap('ds.store.operatingRes',res);
dsupWrap('ds.store.operaingPrintDir',printDir);
dsupWrap('ds.store.operatingChainLength',idx);
dsupWrap('ds.store.operatingPedroHog',pedroHog);
dsupWrap('ds.store.operatingHogStepSize',params.hogStepSize);
dsupWrap('ds.store.operatingWhitenMu',whitenMu);
dsupWrap('ds.store.operatingWhitenRT',whitenRT);
dsupWrap('ds.store.operatingExtRate',params.extRate);
dssaveAndWait;
dsrundistributed('dumpChain(dsidx);',length(ds.store.operaingIdx));
imGrad=ds.res.imGrad;
dsdelete('ds.res.imGrad');


% TODO: remove me
flatHtmlFilename=fullfile(printDir,sprintf('flat_%g.html',idx));
flatImgList=[];
flatImgStr=[];

% make html
htmlFilename=fullfile(printDir,sprintf('%g.html',idx));
N=zeros(length(res),1);
imgList=[];
imgStr=[];
for i = 1:length(res)
    N(i)=size(res{i},1);
    valIdx=~isnan(res{i}(:,13));
    accumApprDist=sum(res{i}(valIdx,13));
    accumContextDist=sum(res{i}(valIdx,17));
    [ignore detRank]=sort(res{i}(:,12),'descend');
    [ignore detRank]=ismember(vec(1:N(i)),detRank);
    
    totalObj=zeros(N(i),1);
    for j = N(i):-1:1
        if ~isnan(res{i}(j,12))
            if j~=1
                totalObj(j)=params.c_proj*max(params.lambda-(res{i}(res{i}(j,15),11)-res{i}(j,11))*res{i}(j,16),0)+params.c_resp*res{i}(j,12)-params.c_appr*res{i}(j,13)-params.c_context*res{i}(j,17);
            else
                totalObj(j)=params.c_resp*res{i}(j,12);
            end
        else
            if j~=1
                totalObj(j)=params.c_proj*max(params.lambda-(res{i}(res{i}(j,15),11)-res{i}(j,11))*res{i}(j,16),0)-params.c_appr*res{i}(j,13)-params.c_context*res{i}(j,17);
            else
                totalObj(j)=0;
            end
        end
    end
    [ignore totalObjRank]=sort(totalObj,'descend');
    [ignore totalObjRank]=ismember(vec(1:N(i)),totalObjRank);
    
    for j = 1:N(i)
        imgList{end+1}=relativepath(fullfile(printDir,sprintf('%g',idx),sprintf('%g',i),sprintf('%g.jpg',imgIdx(j))),printDir);
        if detRank(j)<=params.highlightDetK
            detstr='<font color="green">d=%.3e,dRnk:%g-</font>';
        else
            detstr='d=%.3e,dRnk:%g-';
        end
        if j==1
            imgStr{end+1}=sprintf(['<font color="red">(%g/%g) lambda=%g,cA=%g<br>cP=%g,cR=%g,cC=%g</font><br>'...
                '<font color="blue">score=%.3f<br>apprS=%.3f<br>contextS=%.3f</font><br>img:%g(%g)<br>obj:%g, objR:%g-<br>r=%.3e<br>' detstr '<br>[h:%g w:%g s:%g]<br>[%g %g %g %g | %g]<br>'...
                'parNIdx=%g,isPS=%g'],i,length(res),params.lambda,params.c_appr,...
                params.c_proj,params.c_resp,params.c_context,score(i),accumApprDist,accumContextDist,imgIdx(j),j,totalObj(j),totalObjRank(j),res{i}(j,11),res{i}(j,12),detRank(j),res{i}(j,8)-res{i}(j,6)+1,res{i}(j,7)-res{i}(j,5)+1,...
                res{i}(j,10),res{i}(j,5),res{i}(j,6),res{i}(j,7),res{i}(j,8),res{i}(j,9),res{i}(j,15),res{i}(j,16));
        else
            imgStr{end+1}=sprintf(['img:%g(%g)<br>obj:%g, objR:%g-<br>r=%.3e<br>' detstr '<br>ad2p=%g<br>cd2p=%g<br>[h:%g w:%g s:%g]<br>[%g %g %g %g | %g]<br>parNIdx=%g,isPS=%g'],...
                imgIdx(j),j,totalObj(j),totalObjRank(j),res{i}(j,11),res{i}(j,12),detRank(j),res{i}(j,13),res{i}(j,17),res{i}(j,8)-res{i}(j,6)+1,res{i}(j,7)-res{i}(j,5)+1,res{i}(j,10),res{i}(j,5),res{i}(j,6),res{i}(j,7),...
                res{i}(j,8),res{i}(j,9),res{i}(j,15),res{i}(j,16));
        end
        imgList{end+1}=relativepath(fullfile(printDir,'hog',sprintf('%g',idx),sprintf('%g',i),sprintf('%g.jpg',imgIdx(j))),printDir);
        
        % TODO
        if i==1
            flatImgList{end+1}=relativepath(fullfile(printDir,sprintf('%g',idx),sprintf('%g',i),sprintf('%g.jpg',imgIdx(j))),printDir);
            flatImgStr{end+1}=sprintf('imgIdx=%g,(%g/%g)',imgIdx(j),j,N(i));
        end
        
        imgStr{end+1}=sprintf('grad energy:%g',imGrad{j}(i));
    end
end
displayPatch(htmlFilename,imgList,imgStr,relativepath(fullfile(printDir,sprintf('%g.html',idx-1)),printDir),relativepath(fullfile(printDir,sprintf('%g.html',idx+1)),printDir),2*N,200,200);

% TODO
displayPatch(flatHtmlFilename,flatImgList,flatImgStr,relativepath(fullfile(printDir,sprintf('flat_%g.html',idx-1)),printDir),relativepath(fullfile(printDir,sprintf('flat_%g.html',idx+1)),printDir),18,200,200);

dsdelete('ds.store.operaingIdx');
dsdelete('ds.store.operatingIMDB');
dsdelete('ds.store.operatingRes');
dsdelete('ds.store.operaingPrintDir');
dsdelete('ds.store.operatingChainLength');
dsdelete('ds.store.operatingPedroHog');
dsdelete('ds.store.operatingHogStepSize');
dsdelete('ds.store.operatingWhitenMu');
dsdelete('ds.store.operatingWhitenRT');


function [res ptr score imgIdx pedroHog]=backtrack(structData,isChainNMS)
% format of res
% [1-4 bbox, 5-8 bbox on feat, 9 layer index, 10 layer scale, 11 ranker resp, 
%           12 detector resp, 13 appearance dist to parent, 14 nodeIdx, 15 parentNodeIdx, 16 isParentStronger 17 context dist to parent]


imgIdx=cellfun(@(x)x.imgIdx,structData);
pedroHog=cellfun(@(x)x.pedroHog,structData,'uniformoutput',false);
maskNeighbor=2;

optStructData=structData;
if isChainNMS
    % The overlapping prunning strategy
    topK=500;
    score=zeros(topK,1);
    pathStart=zeros(topK,4);
    res=cell(topK,1);
    ptr=cell(topK,1);
    for pp=1:topK
        map=optStructData{1}.score;
        tmpPathStart=[];
        tmpScore=[];
        for i = 1:length(map)
            for j =1:length(map{i})
                if isempty(map{i}{j}) || sum(isinf(vec(map{i}{j})))==numel(map{i}{j})
                    continue;
                end
                mapInstance=map{i}{j};
                [maxVal maxIdx]=max(vec(mapInstance));
                indptr=maxIdx;
                [hptr,wptr]=ind2sub([size(mapInstance,1) size(mapInstance,2)],indptr);
                if isempty(hptr)
                    continue;
                end
                tmpPathStart=[tmpPathStart;hptr wptr i*ones(length(hptr),1) j*ones(length(hptr),1)];
                idx=sub2ind(size(map{i}{j}),hptr,wptr);
                tmpScore=[tmpScore;map{i}{j}(idx)];
            end
        end
        
        if isempty(tmpScore)
            fprintf('we do not have any possible route at iter %g, we should stop\n',pp);
            res=res(1:pp-1);
            ptr=ptr(1:pp-1);
            score=score(1:pp-1);
            return;
        end
        
        [maxVal maxIdx]=max(tmpScore);
        pathStart(pp,:)=tmpPathStart(maxIdx,:);
        score(pp)=tmpScore(maxIdx);

        % launch backtrack driver given starting point
        [res{pp} ptr{pp}]=backtrackRunner(pathStart(pp,:),optStructData);
        if isempty(res{pp}) || isempty(ptr{pp})
            res=res(1:pp-1);
            ptr=ptr(1:pp-1);
            score=score(1:pp-1);
            return;
        end
        
        optStructData=maskoutPath(optStructData,ptr{pp},maskNeighbor);
    end
    
    % % NMS over the chains
    % img_ov_thresh=0.7;
    % chain_ov_thresh=0.9;
    % [res valIdx]=chainNMS(res,score,img_ov_thresh,chain_ov_thresh);
    % ptr=ptr(valIdx);
    % score=score(valIdx);
    
else
    map=structData{1}.score;
    %below is the standard local maximum search strategy
    h=fspecial('gaussian',4,4);
    thresh=-inf;
    pathStart=[];
    score=[];
    for i = 1:length(map)
        for j =1:length(map{i})
            if isempty(map{i}{j}) || sum(isinf(vec(map{i}{j})))==numel(map{i}{j}) || sum(isnan(vec(map{i}{j})))==numel(map{i}{j})
                continue;
            end
            mapInstance=map{i}{j};
            NAmask=isinf(mapInstance) | isnan(mapInstance);
            mapInstance(NAmask)=min(mapInstance(~NAmask))-max(abs(mapInstance(~NAmask)));
            [hptr,wptr]=find(imregionalmax(mapInstance,8) & (mapInstance>=thresh));
            hptr=vec(hptr);wptr=vec(wptr);
            indptr=sub2ind([size(mapInstance,1) size(mapInstance,2)],hptr,wptr);
            pruneIdx=NAmask(indptr);
            hptr(pruneIdx)=[];
            wptr(pruneIdx)=[];
            if isempty(hptr)
                continue;
            end
            pathStart=[pathStart;hptr wptr i*ones(length(hptr),1) j*ones(length(hptr),1)];
            idx=sub2ind(size(map{i}{j}),hptr,wptr);
            score=[score;vec(map{i}{j}(idx))];
        end
    end
    res=cell(size(pathStart,1),1);
    ptr=cell(size(pathStart,1),1);
    for i = 1:size(pathStart,1)
        [res{i} ptr{i}]=backtrackRunner(pathStart(i,:),structData);
    end
    
    [sortVal sortIdx]=sort(score,'descend');
    K=min(length(sortIdx),5);
    res=res(sortIdx(1:K));
    ptr=ptr(sortIdx(1:K));
    score=score(sortIdx(1:K));
end


function structData=initMaskout(structData,pathPtr,maskNeighbor)

nodeIdx=cellfun(@(x)x.nodeIdx,structData);
structData{1}=rmfield(structData{1},'score');
for i = length(structData):-1:2
    structData{i}=rmfield(structData{i},'score');
    if isfield(structData{i},'lh')
        structData{i}=rmfield(structData{i},'lh');
    end
    if isfield(structData{i},'lw')
        structData{i}=rmfield(structData{i},'lw');
    end
    if isfield(structData{i},'lk')
        structData{i}=rmfield(structData{i},'lk');
    end
    if isfield(structData{i},'lad')
        structData{i}=rmfield(structData{i},'lad');
    end
    
    % mask out operation
    corrIdx=nodeIdx==structData{i}.parent;
    childPoint=pathPtr(i,:);
    childSW=structData{i}.sw;
    childIntLength=size(structData{i}.pyramidSize,1);
    
    parentPoint=pathPtr(corrIdx,:);
    parentSW=structData{corrIdx}.sw;
    parentIntLength=size(structData{corrIdx}.pyramidSize,1);
    
    
    
    % childHMark=structData{i}.hMark{childPoint(3)}{childPoint(4)};
    % childWMark=structData{i}.wMark{childPoint(3)}{childPoint(4)};
    % 
    % maskH=childPoint(1)-maskNeighbor:childPoint(1)+maskNeighbor;
    % maskW=childPoint(2)-maskNeighbor:childPoint(2)+maskNeighbor;
    % expMaskH=vec(repmat(vec(maskH),[1 length(maskW)]));
    % expMaskW=vec(repmat(maskW,[length(maskH) 1]));
    % validIdx=expMaskH>=1 & expMaskH<=length(childHMark) & expMaskW>=1 & expMaskW<=length(childWMark);
    % %childTemplatePtr=sub2ind([length(childHMark) length(childWMark)],childPoint(1),childPoint(2));
    % childTemplatePtr=sub2ind([length(childHMark) length(childWMark)],expMaskH(validIdx),expMaskW(validIdx));
    % 
    % parentHMark=structData{corrIdx}.hMark{parentPoint(3)}{parentPoint(4)};
    % parentWMark=structData{corrIdx}.wMark{parentPoint(3)}{parentPoint(4)};
    % 
    % maskH=parentPoint(1)-maskNeighbor:parentPoint(1)+maskNeighbor;
    % maskW=parentPoint(2)-maskNeighbor:parentPoint(2)+maskNeighbor;
    % expMaskH=vec(repmat(vec(maskH),[1 length(maskW)]));
    % expMaskW=vec(repmat(maskW,[length(maskH) 1]));
    % validIdx=expMaskH>=1 & expMaskH<=length(parentHMark) & expMaskW>=1 & expMaskW<=length(parentWMark);
    % %parentTemplatePtr=sub2ind([length(parentHMark) length(parentWMark)],parentPoint(1),parentPoint(2));
    % parentTemplatePtr=sub2ind([length(parentHMark) length(parentWMark)],expMaskH(validIdx),expMaskW(validIdx));
    % 
    % idx=sub2ind([childIntLength size(childSW,1) parentIntLength size(parentSW,1)],childPoint(4),childPoint(3),parentPoint(4),parentPoint(3));
    % structData{i}.toParentObj{idx}(parentTemplatePtr,:)=-inf;
    % structData{i}.toParentObj{idx}(:,childTemplatePtr)=-inf;
    
    
    parentHMark=structData{corrIdx}.hMark{parentPoint(3)}{parentPoint(4)};
    parentWMark=structData{corrIdx}.wMark{parentPoint(3)}{parentPoint(4)};
    maskH=parentPoint(1)-maskNeighbor:parentPoint(1)+maskNeighbor;
    maskW=parentPoint(2)-maskNeighbor:parentPoint(2)+maskNeighbor;
    expMaskH=vec(repmat(vec(maskH),[1 length(maskW)]));
    expMaskW=vec(repmat(maskW,[length(maskH) 1]));
    validIdx=expMaskH>=1 & expMaskH<=length(parentHMark) & expMaskW>=1 & expMaskW<=length(parentWMark);
    %parentTemplatePtr=sub2ind([length(parentHMark) length(parentWMark)],parentPoint(1),parentPoint(2));
    parentTemplatePtr=sub2ind([length(parentHMark) length(parentWMark)],expMaskH(validIdx),expMaskW(validIdx));
    for childIntIdx=1:childIntLength
        idx=sub2ind([childIntLength size(childSW,1) parentIntLength size(parentSW,1)],childIntIdx,childPoint(3),parentPoint(4),parentPoint(3));
        structData{i}.toParentObj{idx}(parentTemplatePtr,:)=-inf;
    end
    
    
    childHMark=structData{i}.hMark{childPoint(3)}{childPoint(4)};
    childWMark=structData{i}.wMark{childPoint(3)}{childPoint(4)};
    maskH=childPoint(1)-maskNeighbor:childPoint(1)+maskNeighbor;
    maskW=childPoint(2)-maskNeighbor:childPoint(2)+maskNeighbor;
    expMaskH=vec(repmat(vec(maskH),[1 length(maskW)]));
    expMaskW=vec(repmat(maskW,[length(maskH) 1]));
    validIdx=expMaskH>=1 & expMaskH<=length(childHMark) & expMaskW>=1 & expMaskW<=length(childWMark);
    childTemplatePtr=sub2ind([length(childHMark) length(childWMark)],expMaskH(validIdx),expMaskW(validIdx));
    for parentIntIdx=1:parentIntLength
        idx=sub2ind([childIntLength size(childSW,1) parentIntLength size(parentSW,1)],childPoint(4),childPoint(3),parentIntIdx,parentPoint(3));
        structData{i}.toParentObj{idx}(:,childTemplatePtr)=-inf;
    end
end




function structData=maskoutPath(structData,pathPtr,maskNeighbor)

structData=initMaskout(structData,pathPtr,maskNeighbor);

nodeIdx=cellfun(@(x)x.nodeIdx,structData);
for k = length(structData):-1:2
    child=structData{k};
    parent=structData{nodeIdx==child.parent};
    
    parentIntLength=size(parent.pyramidSize,1);
    parentIntSize=parent.pyramidSize;
    parentSW=parent.sw;
    parentStepsize=parent.stepsize;
    total_parentHMark=parent.hMark;
    total_parentWMark=parent.wMark;
    
    childIntLength=size(child.pyramidSize,1);
    childIntSize=child.pyramidSize;
    childSW=child.sw;
    childStepsize=child.stepsize;
    MC=child.M;
    NC=child.N;
    total_apprDist=child.toParentApprDist;
    total_overallObj=child.toParentObj;
    
    if isfield(child,'score')
        childHaveChildren=true;
        childScore=child.score;
    else
        childHaveChildren=false;
        childScore=[];
    end
    
    % add scores on
    for p = 1:size(parentSW,1)
        total_score{p}=cell(parentIntLength,1);
        total_lh{p}=cell(parentIntLength,1);
        total_lw{p}=cell(parentIntLength,1);
        total_Ik{p}=cell(parentIntLength,1);
        total_lad{p}=cell(parentIntLength,1);
        for i = 1:parentIntLength
            if parentIntSize(i,1)<parentSW(p,1) || parentIntSize(i,2)<parentSW(p,2)
                continue;
            end
            respSize=genSize(parentIntSize(i,:),parentSW(p,:),parentStepsize(i));
            respH=respSize(1);
            respW=respSize(2);
            score=-inf(respH,respW,size(childSW,1)*childIntLength);
            lh=nan(respH,respW,size(childSW,1)*childIntLength);
            lw=nan(respH,respW,size(childSW,1)*childIntLength);
            lad=nan(respH,respW,size(childSW,1)*childIntLength);
            layer0=reshape(1:respH*respW,[respH respW]);

            for q = 1:size(childSW,1)
                for j = 1:childIntLength
                    layerIdx=(q-1)*childIntLength+j;
                    if childIntSize(j,1)<childSW(q,1) || childIntSize(j,2)<childSW(q,2) || sum(parentSW(p,:)==childSW(q,:))~=size(parentSW,2)
                        continue;
                    end
                    idx=sub2ind([childIntLength size(childSW,1) parentIntLength size(parentSW,1)],j,q,i,p);
                    parentHMark=total_parentHMark{p}{i};
                    parentWMark=total_parentWMark{p}{i};
                    apprDist=total_apprDist{idx};
                    overallObj=total_overallObj{idx};
                    if childHaveChildren
                        % child is not a leaf node
                        overallObj=bsxfun(@plus,overallObj,vec(childScore{q}{j})');
                    end
                    [tmpScore maxI]=max(overallObj,[],2);
                    score(:,:,layerIdx)=reshape(tmpScore,[respH respW]);
                    [tmpLh tmpLw]=ind2sub(genSize([MC(q,j) NC(q,j)],childSW(q,:),childStepsize(j)),maxI);
                    lh(:,:,layerIdx)=reshape(tmpLh,[respH respW]);
                    lw(:,:,layerIdx)=reshape(tmpLw,[respH respW]);
                    apprIdx=sub2ind(size(apprDist),vec(1:size(apprDist,1)),maxI);
                    lad(:,:,layerIdx)=reshape(apprDist(apprIdx),[length(parentHMark) length(parentWMark)]); 
                end
            end
            [score maxLayer]=max(score,[],3);
            idx=layer0+respH*respW*(maxLayer-1);
            lh=lh(idx);
            lw=lw(idx);
            lad=lad(idx);

            total_score{p}{i}=score;
            total_lh{p}{i}=lh;
            total_lw{p}{i}=lw;
            total_Ik{p}{i}=maxLayer;
            total_lad{p}{i}=lad;
        end
    end

    parent.score=total_score;
    

    % parentPyramidSize=cellfun(@(x)size(x),parentInt,'uniformoutput',false);
    % parentPyramidSize=cat(1,parentPyramidSize{:});
    % parent.score=total_score;
    % parent.stepsize=parentStepsize;
    % parent.pyramidSize=parentPyramidSize;
    % parent.scales=parentScales;
    % parent.scaledSize=parentScaledSize;
    % parent.resp=parentResp;

    child.score=childScore;
    child.lh=total_lh;
    child.lw=total_lw;
    child.lk=total_Ik;
    child.lad=total_lad;
    
    structData{k}=child;
    structData{nodeIdx==child.parent}=parent;
end






% function [res selectedIdx]=chainNMS(res,score,img_ov_thresh,chain_ov_thresh)
% % the function used to do NMS over a set of chains
% [sortVal sortIdx]=sort(score,'descend');
% selectedIdx=[];
% while ~isempty(sortIdx)
%     current=sortIdx(1);
%     ov=chainOverlapping(res(current),res(sortIdx),img_ov_thresh);
%     affIdx=find(ov>chain_ov_thresh);
%     sortIdx(affIdx)=[];
%     selectedIdx=[selectedIdx;current];
% end
% res=res(selectedIdx);


function os=chainOverlapping(chain1,chain2,thresh)
os=zeros(length(chain1),length(chain2));
for i = 1:length(chain1)
    for j = 1:length(chain2)
        bbox_os=getosmatrix_bb(chain1{i}(:,1:4),chain2{j}(:,1:4));
        bbox_os=diag(bbox_os);
        os(i,j)=sum(bbox_os>thresh)/length(bbox_os);
    end
end




function [res ptr]=backtrackRunner(pathStart,structData)
% [1-4 bbox, 5-8 bbox on feat, 9 layer index, 10 layer scale, 11 ranker resp, 
%           12 detector resp, 13 appearance dist to parent, 14 nodeIdx, 15 parentNodeIdx, 16 isParentStronger 17 context dist to parent]

% the function traces back from root to leave
if sum(isnan(pathStart))>0
    res=[];
    ptr=[];
    return;
end

res=zeros(length(structData),17);
ptr=nan(length(structData),5);

for i = 1:length(structData)
    if i==1
        info=getCoord(pathStart,structData{i}.sw,structData{i}.pyramidSize,structData{i}.stepsize,structData{i}.scales);
        ptr(i,:)=[pathStart structData{i}.nodeIdx];
        if ~isempty(structData{i}.detResp{ptr(i,3)}{ptr(i,4)})
            detResp=structData{i}.detResp{ptr(i,3)}{ptr(i,4)}(ptr(i,1),ptr(i,2));
        else
            detResp=nan;
        end
        res(i,:)=[info structData{i}.resp{ptr(i,3)}{ptr(i,4)}(ptr(i,1),ptr(i,2)) detResp nan structData{i}.nodeIdx structData{i}.parent structData{i}.isParentStronger nan];
    else
        par=find(structData{i}.parent==ptr(:,5));     
        hptr=structData{i}.lh{ptr(par,3)}{ptr(par,4)}(ptr(par,1),ptr(par,2));
        wptr=structData{i}.lw{ptr(par,3)}{ptr(par,4)}(ptr(par,1),ptr(par,2));
        layerptr=structData{i}.lk{ptr(par,3)}{ptr(par,4)}(ptr(par,1),ptr(par,2));
        apprDist=structData{i}.lad{ptr(par,3)}{ptr(par,4)}(ptr(par,1),ptr(par,2));
        contextDist=structData{i}.lcd{ptr(par,3)}{ptr(par,4)}(ptr(par,1),ptr(par,2));
        [v u]=ind2sub([size(structData{i}.pyramidSize,1) size(structData{i}.sw,1)],layerptr);
        if isnan(u)||isnan(v)||isnan(hptr)||isnan(wptr)
            res=[];
            ptr=[];
            return;
        end
        info=getCoord([hptr wptr u v],structData{i}.sw,structData{i}.pyramidSize,structData{i}.stepsize,structData{i}.scales);
        ptr(i,:)=[hptr wptr u v structData{i}.nodeIdx];
        if ~isempty(structData{i}.detResp{ptr(i,3)}{ptr(i,4)})
            detResp=structData{i}.detResp{ptr(i,3)}{ptr(i,4)}(ptr(i,1),ptr(i,2));
        else
            detResp=nan;
        end
        res(i,:)=[info structData{i}.resp{ptr(i,3)}{ptr(i,4)}(ptr(i,1),ptr(i,2)) detResp apprDist structData{i}.nodeIdx structData{i}.parent structData{i}.isParentStronger contextDist];
    end
end


function info=getCoord(ptr,sw,pyramidSize,stepsize,scales)
u=ptr(3);
v=ptr(4);
sw=sw(u,:);
pyramidSize=pyramidSize(v,:);
stepsize=stepsize(v);
[ignore hmark wmark]=genSize(pyramidSize,sw,stepsize);
hptr=hmark(ptr(1));
wptr=wmark(ptr(2));
pyramidBbox=[wptr hptr wptr+sw(2)-1 hptr+sw(1)-1 v scales(v)];
info=[pyramidBbox(1:4)/scales(v) pyramidBbox];

function [size hmark wmark]=genSize(mapSize,sw,stepsize)
hmark=1:stepsize:mapSize(1)-sw(1)+1;
wmark=1:stepsize:mapSize(2)-sw(2)+1;
size(1)=length(hmark);
size(2)=length(wmark);









